/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/
/* Generic headers */
#include <stdint.h>
#include <stdbool.h>

/* BFE related headers */
#include "bal_data.h"

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Local Typedef definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global variables
 *********************************************************************************************************************/

/* Extended configuration structure */
const st_raa489220_ext_cfg_t g_bfe0_ext_cfg =
{
 .r_shunt_charge_mohm = 1000.0f,                   // Set charge current shunt resistance in mohms!
 .r_shunt_discharge_mohm = 100.0f,                // Set discharge current shunt resistance in mohms!

 .thold_overvolt_lockout_v = 4.30f,             // Set the overvoltage lockout threshold in Volts! (3.0-4.5V, 100mV)
 .thold_overvolt_v = 4.25f,                     // Set the overvoltage threshold in Volts! (3.23-4.5V, 10mV)
 .thold_undervolt_v = 2.70f,                    // Set the undervoltage threshold in Volts! (1.5-3.0V, 100mV)
 .thold_undervolt_lockout_v = 2.00f,            // Set the undervoltage lockout threshold in Volts! (1.5-3.0V, 100mV)
 .thold_delta_cell_overvolt_v = 0.5f,           // Set the delta cell overvoltage threshold in Volts! (0.5-2.0V, 100mV)

 .hyst_overovltage = BFE_CELL_OV_HYST_100MV,    // Set the cell overvoltage hysteresis!
 .hyst_undervltage = BFE_CELL_UV_HYST_300MV,    // Set the cell undervoltage hysteresis!

 .thold_charge_over_temp = BFE_COT_THLD_0_294V,     // Set the charge over-temperature threshold!
 .thold_charge_under_temp = BFE_CUT_THLD_1_198V,    // Set the charge under-temperature threshold!
 .thold_discharge_over_temp = BFE_DOT_THLD_0_150V,  // Set the discharge over-temperature threshold!
 .thold_discharge_under_temp = BFE_DUT_THLD_1_622V, // Set the discharge under-temperature threshold!

 .thold_charge_overcurrent_a = 0.050f,              // Set the charge overcurrent threshold in Amps!
 .thold_discharge_overcurrent_a = 0.500f,           // Set the discharge overcurrent threshold in Amps!
 .delay_ch_d_oc = BFE_OC_USE_FAULT_DELAY,           // Set discharge/charge overcurrent delay option!
 .thold_sc = BFE_SC_THLD_200MV,                     // Set the short-circuit detection threshold!
 .delay_sc = BFE_SC_DELAY_1MS,                      // Set the short-circuit current detection delay!
 .ieoc_threshold = BFE_IEOC_THLD_0_65MV,            // Set the current end of charge threshold!
 .discharge_overcurrent_disable = false,            // Make 'true' to disable discharge overcurrent protection!

 .comm_timeout = BFE_COMM_TO_OFF,                   // Set communication timeout!

 .self_test_enable = true,                          // Include or not self-testing in single scan sequence!
 .open_wire_test_enable = true,                     // Include or not open wire test in the self-test loop!
 .fuse_blow_enable = false,                         // Enable or disable fuse blowing!

 .scan_delay = BFE_SCAN_DELAY_0_1S,                 // Set the scan delay relevant to DOC/CHOC detection delay!
 .fault_delay = BFE_FAULT_DELAY_1S,                 // Set the fault delay relevant to DOC/CHOC detection delay!
 .lp_mode_settings = BFE_LP_MODE_STRONG_REG,        // Set the Low Power Mode Settings!

 .alert_pin_cfg = BFE_ALRT_NORMAL,                  // Set the ALERT pin internal connection!

 .pin_alert = BSP_IO_PORT_01_PIN_05,                // Set MCU pin connected to the ALERT pin of the BFE
 .pin_reset = BSP_IO_PORT_00_PIN_14,                // Set MCU pin connected to the THERM2 pin of the BFE
};

/* Set which cells exist in the battery pack! */
const uint16_t g_bfe0_cells_cfg = 0x03FF;               // Do not modify!!!

/* Set which external temperature inputs are used! */
const uint16_t g_bfe0_ext_temps_cfg = BFE_THERM_1_2;    // Select which thermistors will be monitored

/* Configuration structure */
const st_bfe_cfg_t g_bfe0_cfg =
{
 .p_cells_select = &g_bfe0_cells_cfg,                   // Do not modify!!!
 .p_temps_select = &g_bfe0_ext_temps_cfg,               // Do not modify!!!
 .peripheral_type = BFE_COMMUNICATION_INTERFACE_I2C,    // Do not modify!!!
 .driver_cfg = BFE_DRIVER_LOW_SIDE,                     // Do not modify!!!
 .fet_cfg = BFE_FET_CONFIG_PARALLEL,                    // Select FETs configuration!
 .p_extend = &g_bfe0_ext_cfg,                           // Do not modify!!!
};

/* Extended control structure */
st_raa489220_ext_ctrl_t g_bfe0_ext_ctrl =
{
 .i2c_slave_address = 0,
 .weak_regulator = false,

 /* Timings structure */
 .timings =
 {
  .reset_wait_time_ms = 0,                              // Do not modify!!!
  .reset_therm2_hold_time_ms = 0,                       // Do not modify!!!
 },
};

/* BFE IC information data structure */
st_raa489220_info_t g_bfe0_info = {0};  // Do not modify!!!

/* Control structure */
st_bfe_ctrl_t g_bfe0_ctrl =
{
 .is_initialized = false,               // Do not modify!!!
 .is_low_power = false,                 // Do not modify!!!
 .is_balancing = false,                 // Do not modify!!!
 .is_cont_scanning = false,             // Do not modify!!!
 .is_fault_detected = false,            // Do not modify!!!
 .p_bfe_info = &g_bfe0_info,            // Do not modify!!!
 .p_bfe_regs = &g_raa489220_registers,  // Do not modify!!!
 .p_cfg = &g_bfe0_cfg,                  // Do not modify!!!
 .p_extend = &g_bfe0_ext_ctrl,          // Do not modify!!!
};

/* Instance structure to use BFE module. */
const bfe_instance_t g_bfe0 =
{ .p_ctrl = &g_bfe0_ctrl, .p_cfg = &g_bfe0_cfg, .p_api = &g_bfe_on_bfe};

/**********************************************************************************************************************
 Private (static) variables and functions
 *********************************************************************************************************************/
