/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2021 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/******************************************************************************************************************//**
 * @file          r_bfe_cfg.h
 * @Version       1.0
 * @brief         Contains configuration settings (macros) for the battery front end library functions.
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 01.06.2022 1.00     First Release
 *********************************************************************************************************************/

#ifndef R_BFE_CFG_H_
#define R_BFE_CFG_H_

/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/
/* Generic headers */
#include <stdint.h>
#include <stdbool.h>

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/

/*
 *   |        PMOD description and jumper settings         |
 *   |-----------------------------------------------------|
 *   | Signal | R7FA4W1AD2CNG | RTKAPMODDONGLEEV1Z         |
 *   |--------+---------------+----------------------------|
 *   | N/C    |               | J2-1:  JB1(OPEN)           |
 *   | SCL    | P100          | J2-2:  JB2(1,2) J3(CLOSED) |
 *   | SDA    | P101          | J2-3:  JB3(2,3) J4(CLOSED) |
 *   | N/C    |               | J2-4:  JB4(OPEN)           |
 *   | GND    | VSS           | J2-5:                      |
 *   | V3P3   |               | J2-6:  J14(OPEN)           |
 *   | ALERT  | P105          | J2-7:  JB7(1,2)            |
 *   | RESET  | P014          | J2-8:  JB8(2,3)            |
 *   | N/C    |               | J2-9:  JB9(OPEN)           |
 *   | HVGPIO | P501          | J2-10: JB10(1,2)           |
 *   | GND    | VSS           | J2-11:                     |
 *   | V3P3   |               | J2-12: J14(OPEN)           |
 *   |------------------------+----------------------------|
 */

/** '0' - Do not use/ '1' - Use (Recommended) CRC check in i2c communication. */
#define BFE_I2C_CRC_USE                     (1)

/** Functions check input parameters: '0' - Disable/ '1' - Enable (Recommended). */
#define BFE_CFG_PARAM_CHECKING_EN           (1)

/** Register verification after write command: '0' - Disable/ '1' - Enable (Recommended). */
#define BFE_CFG_REG_WRITE_VERIFY_EN         (1)

/**********************************************************************************************************************
 Global Typedef definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 External global variables
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global functions
 *********************************************************************************************************************/

#endif /* R_BFE_CFG_H_ */
