/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2021 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/******************************************************************************************************************//**
 * @file          r_raa489220.c
 * @version       1.00
 * @brief         Contains the implementations of the BFE api. The functions provide access to the
 * full functionality of RAA489220.
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 01.06.2022 1.00     First Release.
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Includes   <System Includes>, "Project Includes"
 *********************************************************************************************************************/
/* Generic headers */
#include <stdint.h>
#include <stdbool.h>

/* BFE related headers */
#include <bfe/r_raa489220.h>

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Local Typedef definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global variables
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Private (static) variables and functions
 *********************************************************************************************************************/

/** BFE implementation of BFE interface. */
const bfe_api_t g_bfe_on_bfe =
{
 .p_initialize               = R_RAA489220_Init,
 .p_deinitialize             = R_RAA489220_Deinit,
 .p_setup                    = R_RAA489220_Setup,
 .p_reset                    = R_RAA489220_Reset,
 .p_modeSet                  = R_RAA489220_ModeSet,
 .p_modeRead                 = R_RAA489220_ModeRead,
 .p_commTest                 = R_RAA489220_CommTest,
 .p_selfDiagnostic           = R_RAA489220_SelfDiag,
 .p_memoryCheck              = R_RAA489220_MemCheck,
 .p_vPackGet                 = R_RAA489220_VPackGet,
 .p_iPackGet                 = R_RAA489220_IPackGet,
 .p_vCellsGet                = R_RAA489220_VCellsGet,
 .p_temperaturesGet          = R_RAA489220_Temps,
 .p_allGet                   = R_RAA489220_AllGet,
 .p_otherGet                 = R_RAA489220_SysScanGet,
 .p_faultsAllRead            = R_RAA489220_FaultsAllRead,
 .p_faultsCheck              = R_RAA489220_FaultsCheck,
 .p_faultsAllClear           = R_RAA489220_FaultsAllClear,
 .p_cellBalanceControl       = R_RAA489220_CellBalanceCtrl,
 .p_isCellBalancing          = R_RAA489220_IsCellBalancing,
 .p_continuousScanControl    = R_RAA489220_ContScanCtrl,
 .p_wdControl                = R_RAA489220_WatchdogCtrl,
 .p_fetControl               = R_RAA489220_FETsCtrl,
 .p_gpioControl              = R_RAA489220_HVGPIOCtrl,
 .p_registerRead             = R_RAA489220_RegisterRead,
 .p_registerWrite            = R_RAA489220_RegisterWrite,
};

/* RAA489220 registers' bank */
st_raa489220_registers_t g_raa489220_registers =
{
 .pack_voltage      = {.address = BFE_PACK_VOLTAGE,      .type = READ_ONLY,  .value = 0x00, .def_value = BFE_PACK_VOLTAGE_DEF},
 .cell_max_voltage  = {.address = BFE_CELL_MAX_VOLTAGE,  .type = READ_ONLY,  .value = 0x00, .def_value = BFE_CELL_MAX_VOLTAGE_DEF},
 .cell_min_voltage  = {.address = BFE_CELL_MIN_VOLTAGE,  .type = READ_ONLY,  .value = 0x00, .def_value = BFE_CELL_MIN_VOLTAGE_DEF},
 .thermistor_1      = {.address = BFE_THERMISTOR_1,      .type = READ_ONLY,  .value = 0x00, .def_value = BFE_THERMISTOR_1_DEF},
 .thermistor_2      = {.address = BFE_THERMISTOR_2,      .type = READ_ONLY,  .value = 0x00, .def_value = BFE_THERMISTOR_2_DEF},
 .discharge_current = {.address = BFE_DISCHARGE_CURRENT, .type = READ_ONLY,  .value = 0x00, .def_value = BFE_DISCHARGE_CURRENT_DEF},
 .charge_current    = {.address = BFE_CHARGE_CURRENT,    .type = READ_ONLY,  .value = 0x00, .def_value = BFE_CHARGE_CURRENT_DEF},
 .faults            = {.address = BFE_FAULTS,            .type = READ_ONLY,  .value = 0x00, .def_value = BFE_FAULTS_DEF},
 .masks_status      = {.address = BFE_MASKS_AND_STATUS,  .type = READ_WRITE, .value = 0x00, .def_value = BFE_MASKS_AND_STATUS_DEF},
 .system_config_1   = {.address = BFE_SYSTEM_CONFIG_1,   .type = READ_WRITE, .value = 0x00, .def_value = BFE_SYSTEM_CONFIG_1_DEF},
 .measure_select    = {.address = BFE_MEASURE_SELECT,    .type = READ_WRITE, .value = 0x00, .def_value = BFE_MEASURE_SELECT_DEF},
 .system_config_2   = {.address = BFE_SYSTEM_CONFIG_2,   .type = READ_WRITE, .value = 0x00, .def_value = BFE_SYSTEM_CONFIG_2_DEF},
 .system_config_3   = {.address = BFE_SYSTEM_CONFIG_3,   .type = READ_WRITE, .value = 0x00, .def_value = BFE_SYSTEM_CONFIG_3_DEF},
 .ov_eoc            = {.address = BFE_OV_AND_EOC,        .type = READ_WRITE, .value = 0x00, .def_value = BFE_OV_AND_EOC_DEF},
 .dvcov_uv          = {.address = BFE_DV_CELL_OV_AND_UV, .type = READ_WRITE, .value = 0x00, .def_value = BFE_DV_CELL_OV_AND_UV_DEF},
 .scc_ot_ut         = {.address = BFE_SCC_OT_AND_UT,     .type = READ_WRITE, .value = 0x00, .def_value = BFE_SCC_OT_AND_UT_DEF},
 .current           = {.address = BFE_CURRENT,           .type = READ_WRITE, .value = 0x00, .def_value = BFE_CURRENT_DEF},
 .die_revision      = {.address = BFE_DIE_REVISION,      .type = READ_ONLY,  .value = 0x00, .def_value = BFE_DIE_REVISION_DEF},
 .manufacturing_id  = {.address = BFE_MANUFACTURING_ID,  .type = READ_ONLY,  .value = 0x00, .def_value = BFE_MANUFACTURING_ID_DEF},
 .device_id         = {.address = BFE_DEVICE_ID,         .type = READ_ONLY,  .value = 0x00, .def_value = BFE_DEVICE_ID_DEF},
 .user_sel_mode     = {.address = BFE_USER_SELECT_MODE,  .type = READ_WRITE, .value = 0x00, .def_value = BFE_USER_SELECT_MODE_DEF},
 .cmnd_fault_stat   = {.address = BFE_FAULT_AND_STATUS,  .type = COMMAND,    .value = 0x00, .def_value = BFE_FAULT_AND_STATUS_DEF},
 .cmnd_measurements = {.address = BFE_READ_MEASUREMENTS, .type = COMMAND,    .value = 0x00, .def_value = BFE_READ_MEASUREMENTS_DEF},
 .cmnd_i_pack       = {.address = BFE_I_PACK,            .type = COMMAND,    .value = 0x00, .def_value = BFE_I_PACK_DEF},
};

/* Event flags */
static volatile i2c_master_event_t s_i2c_callback_event = RESET_VALUE;
static volatile bool               s_alert_event_flag   = false;

/* Data buffers */
static uint8_t  s_i2c_buffer[BFE_I2C_DATA_LGTH]       = {0};
static uint16_t s_data_buffer[BFE_MAX_READ_DATA_LGTH] = {0};

static e_bfe_err_t bfe_reset_hard (st_bfe_ctrl_t * const p_ctrl);
static e_bfe_err_t bfe_reset_soft (st_bfe_ctrl_t * const p_ctrl);
static e_bfe_err_t bfe_i2c_register_write (st_bfe_ctrl_t * const p_ctrl,
                                           st_raa489220_register_t tar_reg,
                                           uint16_t data);
static e_bfe_err_t bfe_i2c_register_read (st_bfe_ctrl_t * const p_ctrl,
                                          st_raa489220_register_t tar_reg,
                                          uint16_t * const p_data);
static e_bfe_err_t bfe_i2c_block_read (st_bfe_ctrl_t * const p_ctrl,
                                       st_raa489220_register_t consttar_reg,
                                       uint16_t * const p_data);
static fsp_err_t   bfe_i2c_event_validate (void);
static e_bfe_err_t bfe_crc8_calculate (uint8_t * const p_data_buffer, uint32_t length, uint32_t * const p_crc_result);
static e_bfe_err_t bfe_meas_complete_wait (st_bfe_ctrl_t * const p_ctrl, const e_raa489220_meas_select_t selection);
static float       bfe_adc_to_vpack (uint16_t value);
static float       bfe_adc_to_ipack (uint16_t value, float r_shunt_mohm);
static float       bfe_adc_to_vcell (uint16_t value);
static float       bfe_adc_to_vtherm (uint16_t value);
static float       bfe_adc_to_vregpin (uint16_t value);
static float       bfe_adc_to_vbg2ref (uint16_t value);

/* Function Name: R_RAA489220_Init */
/******************************************************************************************************************//**
 * This function initializes the BFE by enabling and configuring the necessary peripheral modules of the MCU.
 * It modifies the 'p_ctrl->is_initialized', 'p_ctrl->is_low_power' and 'p_ctrl->is_cont_scanning' flags in p_ctrl.
 *
 * This function performs the following tasks:
 * - Parameter checking and error conditions processing.
 * - BFE timings calculation.
 * - Initialize a I2C interface to connect to the BFE.
 * - Initialize a CRC interface.
 * - Initialize interrupt modules.
 * - Reset the BFE.
 * - Read BFE die revision, manufacturing id and device id.
 *
 * @pre Please, perform an appropriate peripheral communication module setup, according to the BFE datasheet!
 * @warning This function does not check the communication module settings!
 * @param[in]       p_ctrl                              Pointer to the control structure
 * @param[in]       p_cfg                               Pointer to the configuration structure
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_COMM_UNSUP_INTERFACE        The selected communication interface is unsupported
 * @retval          BFE_ERR_NONSUPPORTED_MODE           The selected configuration setting is unsupported
 * @retval          BFE_ERR_FSP                         Error in the FSP layer
 * @retval          BMS_ERR_...                         Inherit from bfe_reset_hard()
 * @retval          BMS_ERR_...                         Inherit from bfe_i2c_register_read()
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Init (st_bfe_ctrl_t * const p_ctrl, st_bfe_cfg_t const * const p_cfg)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status
    fsp_err_t   fsp_err = FSP_SUCCESS; // Error status

    st_raa489220_ext_ctrl_t  * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;
    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    st_raa489220_info_t *      p_bfe_info = (st_raa489220_info_t *) p_ctrl->p_bfe_info;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_cfg,  BFE_ERR_INVALID_POINTER);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Check configuration settings. */
    BFE_ERROR_RETURN(BFE_COMMUNICATION_INTERFACE_I2C == p_cfg->peripheral_type, BFE_ERR_COMM_UNSUP_INTERFACE);
    BFE_ERROR_RETURN(BFE_DRIVER_LOW_SIDE == p_cfg->driver_cfg, BFE_ERR_UNSUPPORTED_MODE);

    /* Calculate BFE timings */
    p_ext_ctrl->timings.reset_wait_time_ms        = BFE_START_UP_DELAY_MS;
    p_ext_ctrl->timings.reset_therm2_hold_time_ms = BFE_THERM2_RST_HOLD_TIME_MS;
    p_ext_ctrl->timings.busy_bit_timeout_us       = BFE_BUSY_BIT_READ_TIMEOUT_US;

    /* Initialize the IIC module */
    fsp_err = g_i2c_raa489220.p_api->open(&g_i2c_raa489220_ctrl, &g_i2c_raa489220_cfg);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

#if BFE_I2C_CRC_USE // I2C communication with CRC.

    /* Open CRC driver. */
    fsp_err = g_crc8.p_api->open(&g_crc8_ctrl, &g_crc8_cfg);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    p_ext_ctrl->i2c_slave_address = BFE_I2C_ADDRESS_CRC;

#else // I2C communication without CRC.

    p_ext_ctrl->i2c_slave_address = BFE_I2C_ADDRESS_NO_CRC;

#endif // BFE_I2C_CRC_USE

    /* Open and enable the IRQ module */
    fsp_err = g_alert_irq.p_api->open(&g_alert_irq_ctrl, &g_alert_irq_cfg);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    fsp_err = g_alert_irq.p_api->enable(&g_alert_irq_ctrl);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Reset the BFE */
    bfe_err = bfe_reset_hard(p_ctrl);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Update flags */
    p_ctrl->is_initialized = false;
    p_ctrl->is_low_power = false;
    p_ctrl->is_cont_scanning = false;

    /* Read device information */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->die_revision, &p_bfe_regs->die_revision.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->manufacturing_id, &p_bfe_regs->manufacturing_id.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->device_id, &p_bfe_regs->device_id.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    p_bfe_info->die_revision     = p_bfe_regs->die_revision.value;
    p_bfe_info->manufacturing_id = p_bfe_regs->manufacturing_id.value;
    p_bfe_info->device_id        = p_bfe_regs->device_id.value;

    p_ctrl->is_initialized = true; // Set the initialization complete flag.
    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_Init
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_Deinit */
/******************************************************************************************************************//**
 * This function deinitializes a BFE. It disables the used peripheral modules of the MCU. It modifies the
 * Boolean variables p_ctrl->is_initialized and 'p_ctrl->is_low_power'.
 *
 * This function performs the following tasks:
 * - Parameter checking and error conditions processing.
 * - Deinitialization of the I2C interface.
 * - Deinitialization of the CRC interface.
 * - Deinitialization of the IRQ interfaces.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_FSP                         Error in the FSP layer
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Deinit (st_bfe_ctrl_t * const p_ctrl)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status
    fsp_err_t   fsp_err = FSP_SUCCESS; // Error status

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Deinitialize the IIC module */
    fsp_err = R_IIC_MASTER_Close(&g_i2c_raa489220_ctrl);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

#if BFE_I2C_CRC_USE // I2C communication with CRC.

    /* Open CRC driver. */
    fsp_err = g_crc8.p_api->close(&g_crc8_ctrl);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

#endif // BFE_I2C_CRC_USE

    /* Close the IRQ module */
    fsp_err = g_alert_irq.p_api->close(&g_alert_irq_ctrl);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    p_ctrl->is_initialized = false; // Clear the initialization complete flag.
    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_Deinit
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_Setup */
/******************************************************************************************************************//**
 * This function configures the BFE by writing into all device setup registers. It extracts the necessary data
 * from the control p_ctrl and configuration p_cfg structures.
 *
 * This function performs the following tasks:
 * - Checks configuration parameters and processes errors (optional).
 * - Unlocks registers write.
 * - Sends a multiple register write command to set System Config 1, System Config 2, System Config 3, OV and EOC
 *   Thresholds, DVcellOV UV Thresholds, SCC OT/UT Thresholds and Current Threshold Registers.
 * - Verifies if values are correctly written.
 * - Lock registers write.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       p_cfg                               Pointer to the configuration structure.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_INVALID_CONF                An invalid configuration parameter was detected.
 * @retval          BFE_ERR_WRITE_VERIFY                Write command verification error.
 * @retval          BMS_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BMS_ERR_...                         Inherit from bfe_i2c_register_read().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Setup (st_bfe_ctrl_t * const p_ctrl, st_bfe_cfg_t const * const p_cfg)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_ext_cfg_t   * p_ext_cfg   = (st_raa489220_ext_cfg_t *) p_cfg->p_extend;
    st_raa489220_registers_t * p_bfe_regs  = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    uint16_t * p_temp_inputs               = (uint16_t *) p_cfg->p_temps_select;

    u_raa489220_config1_reg_t    system_config_1 = {0};
    u_raa489220_config2_reg_t    system_config_2 = {0};
    u_raa489220_config3_reg_t    system_config_3 = {0};
    u_raa489220_ov_eoc_th_reg_t  ov_eoc          = {0};
    u_raa489220_dvov_uv_th_reg_t dvcov_uv        = {0};
    u_raa489220_scc_ot_ut_reg_t  scc_ot_ut       = {0};
    u_raa489220_current_th_reg_t current         = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_cfg,  BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Check configuration. */
    BFE_ERROR_RETURN(p_ext_cfg->thold_overvolt_lockout_v >= BFE_VCELL_OVLO_TH_MIN_V, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(p_ext_cfg->thold_overvolt_lockout_v <= BFE_VCELL_OVLO_TH_MAX_V, BFE_ERR_INVALID_CONF);

    BFE_ERROR_RETURN(p_ext_cfg->thold_overvolt_v >= BFE_VCELL_OV_TH_MIN_V, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(p_ext_cfg->thold_overvolt_v <= BFE_VCELL_OV_TH_MAX_V, BFE_ERR_INVALID_CONF);

    BFE_ERROR_RETURN(p_ext_cfg->thold_undervolt_v >= BFE_VCELL_UV_TH_MIN_V, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(p_ext_cfg->thold_undervolt_v <= BFE_VCELL_UV_TH_MAX_V, BFE_ERR_INVALID_CONF);

    BFE_ERROR_RETURN(p_ext_cfg->thold_undervolt_lockout_v >= BFE_VCELL_UVLO_TH_MIN_V, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(p_ext_cfg->thold_undervolt_lockout_v <= BFE_VCELL_UVLO_TH_MAX_V, BFE_ERR_INVALID_CONF);

    BFE_ERROR_RETURN(p_ext_cfg->thold_undervolt_lockout_v < p_ext_cfg->thold_undervolt_v, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(p_ext_cfg->thold_undervolt_v < p_ext_cfg->thold_overvolt_v, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(p_ext_cfg->thold_overvolt_v < p_ext_cfg->thold_overvolt_lockout_v, BFE_ERR_INVALID_CONF);

    BFE_ERROR_RETURN(p_ext_cfg->thold_delta_cell_overvolt_v >= BFE_DVCELL_OV_TH_MIN_V, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(p_ext_cfg->thold_delta_cell_overvolt_v <= BFE_DVCELL_OV_TH_MAX_V, BFE_ERR_INVALID_CONF);

    BFE_ERROR_RETURN(((p_ext_cfg->thold_charge_overcurrent_a / 1000.0f)
                     * p_ext_cfg->r_shunt_charge_mohm) >= BFE_VCS_MIN_V, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(((p_ext_cfg->thold_charge_overcurrent_a / 1000.0f)
                     * p_ext_cfg->r_shunt_charge_mohm) <= BFE_VCS_MAX_V, BFE_ERR_INVALID_CONF);

    BFE_ERROR_RETURN(((p_ext_cfg->thold_discharge_overcurrent_a / 1000.0f)
                     * p_ext_cfg->r_shunt_discharge_mohm) >= BFE_VCS_MIN_V, BFE_ERR_INVALID_CONF);
    BFE_ERROR_RETURN(((p_ext_cfg->thold_discharge_overcurrent_a / 1000.0f)
                     * p_ext_cfg->r_shunt_discharge_mohm) <= BFE_VCS_MAX_V, BFE_ERR_INVALID_CONF);

    /* Unlock thresholds registers write */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_LOCK_KEY_0);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_UNLOCK_KEY_1);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_UNLOCK_KEY_2);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_UNLOCK_KEY_3);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Set System Configure 1 Register */
    system_config_1.value = 0;
    system_config_1.value_b.LCFET_EN = 0; // Ensure Charge FET is off
    system_config_1.value_b.LDFET_EN = 0; // Ensure Discharge FET is off

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_1, system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_1, &p_bfe_regs->system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(system_config_1.value == (p_bfe_regs->system_config_1.value & 0x00FF), BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Set System Configure 2 Register */
    system_config_2.value = 0;

    if(p_ext_cfg->self_test_enable == true)
    {
        system_config_2.value_b.SELF_TEST_EN = 1;
    }
    else
    {
        system_config_2.value_b.SELF_TEST_EN = 0;
    }

    system_config_2.value_b.LP_REG = (uint16_t) ((p_ext_cfg->lp_mode_settings >> 4) & 0x01);

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_2, system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(system_config_2.value == p_bfe_regs->system_config_2.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Set System Configure 3 Register */
    system_config_3.value = 0;

    system_config_3.value_b.COMM_TIMEOUT = p_ext_cfg->comm_timeout;

    if(p_ext_cfg->open_wire_test_enable == true)
    {
        system_config_3.value_b.OW_EN = 1;
    }
    else
    {
        system_config_3.value_b.OW_EN = 0;
    }

    system_config_3.value_b.SLEEP_EN    = (uint16_t) (p_ext_cfg->lp_mode_settings & 0x01);
    system_config_3.value_b.SCAN_DELAY  = (uint16_t) (p_ext_cfg->scan_delay & 0x03);
    system_config_3.value_b.FAULT_DELAY = (uint16_t) (p_ext_cfg->fault_delay & 0x03);

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_3, system_config_3.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_3, &p_bfe_regs->system_config_3.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(system_config_3.value == p_bfe_regs->system_config_3.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Set OV and EOC Thresholds Register */
    ov_eoc.value = 0;
    ov_eoc.value_b.OV_TH   = (uint16_t) (0x7F & ((uint16_t) ((p_ext_cfg->thold_overvolt_v
                                        - BFE_VCELL_OV_TH_MIN_V) / BFE_VCELL_OV_TH_STEP_V)));
    ov_eoc.value_b.OV_HYST = (uint16_t) (0x07 & p_ext_cfg->hyst_overovltage);
    ov_eoc.value_b.OVLO_TH = (uint16_t) (0x0F & ((uint16_t) ((p_ext_cfg->thold_overvolt_lockout_v
                                        - BFE_VCELL_OVLO_TH_MIN_V) / BFE_VCELL_OVLO_TH_STEP_V)));
    ov_eoc.value_b.IEOC    = (uint16_t) (0x03 & p_ext_cfg->ieoc_threshold);

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->ov_eoc, ov_eoc.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->ov_eoc, &p_bfe_regs->ov_eoc.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(ov_eoc.value == p_bfe_regs->ov_eoc.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Set DVcellOV, UV Thresholds Register */
    dvcov_uv.value = 0;
    dvcov_uv.value_b.DVOV_TH = (uint16_t) (0x0F & ((uint16_t) ((p_ext_cfg->thold_delta_cell_overvolt_v
                                          - BFE_DVCELL_OV_TH_MIN_V) / BFE_DVCELL_OV_TH_STEP_V)));
    dvcov_uv.value_b.UV_TH   = (uint16_t) (0x0F & ((uint16_t) ((p_ext_cfg->thold_undervolt_v
                                          - BFE_VCELL_UV_TH_MIN_V) / BFE_VCELL_UV_TH_STEP_V)));

    if(p_cfg->fet_cfg == BFE_FET_CONFIG_SERIES)
    {
        dvcov_uv.value_b.CPWR = 0;
    }
    else if(p_cfg->fet_cfg == BFE_FET_CONFIG_PARALLEL)
    {
        dvcov_uv.value_b.CPWR = 1;
    }
    else
    {
        return BFE_ERR_INVALID_CONF;
    }

    dvcov_uv.value_b.UV_HYST = (uint16_t) (0x07 & p_ext_cfg->hyst_undervltage);
    dvcov_uv.value_b.UVLO_TH = (uint16_t) (0x0F & ((uint16_t) ((p_ext_cfg->thold_undervolt_lockout_v
                                          - BFE_VCELL_UV_TH_MIN_V) / BFE_VCELL_UV_TH_STEP_V)));

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->dvcov_uv, dvcov_uv.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->ov_eoc, &p_bfe_regs->ov_eoc.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(ov_eoc.value == p_bfe_regs->ov_eoc.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Set SCC, OT/UT Thresholds Register */
    scc_ot_ut.value = 0;
    scc_ot_ut.value_b.SCC_DELAY = (uint16_t) (0x03 & p_ext_cfg->delay_sc);
    scc_ot_ut.value_b.DOT       = (uint16_t) (0x07 & p_ext_cfg->thold_discharge_over_temp);
    scc_ot_ut.value_b.DUT       = (uint16_t) (0x07 & p_ext_cfg->thold_discharge_under_temp);
    scc_ot_ut.value_b.THERM_EN  = (uint16_t) (0x03 & *p_temp_inputs);
    scc_ot_ut.value_b.COT       = (uint16_t) (0x07 & p_ext_cfg->thold_charge_over_temp);
    scc_ot_ut.value_b.CUT       = (uint16_t) (0x07 & p_ext_cfg->thold_charge_under_temp);

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->scc_ot_ut, scc_ot_ut.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->scc_ot_ut, &p_bfe_regs->scc_ot_ut.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(scc_ot_ut.value == p_bfe_regs->scc_ot_ut.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Set Current Thresholds Register */
    current.value = 0;

    if(p_ext_cfg->delay_ch_d_oc == BFE_OC_USE_SCAN_DELAY)
    {
        current.value_b.OC_DELAY = 0;
    }
    else if(p_ext_cfg->delay_ch_d_oc == BFE_OC_USE_FAULT_DELAY)
    {
        current.value_b.OC_DELAY = 1;
    }
    else
    {
        return BFE_ERR_INVALID_CONF;
    }

    current.value_b.SCC_TH   = (uint16_t) (0x07 & p_ext_cfg->thold_sc);

    if(p_ext_cfg->discharge_overcurrent_disable == true)
    {
        current.value_b.DOC_TH = BFE_DOC_DISABLE;
    }
    else
    {
        current.value_b.DOC_TH = (uint16_t) (0x3F & ((uint16_t) ((((p_ext_cfg->thold_discharge_overcurrent_a / 1000.0f)
                                            * p_ext_cfg->r_shunt_discharge_mohm) - BFE_VCS_MIN_V) / BFE_VCS_STEP_V)));
    }

    current.value_b.COC_TH = (uint16_t) (0x3F & ((uint16_t) ((((p_ext_cfg->thold_charge_overcurrent_a / 1000.0f)
                                        * p_ext_cfg->r_shunt_charge_mohm) - BFE_VCS_MIN_V) / BFE_VCS_STEP_V)));

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->current, current.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->current, &p_bfe_regs->current.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(current.value == p_bfe_regs->current.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Lock thresholds registers write */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_LOCK_KEY_0);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_Setup
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_Reset */
/******************************************************************************************************************//**
 * This function resets the Battery Front End. Several predefined reset options can be set with the type input
 * parameter. It modifies the 'p_ctrl->is_low_power' and 'p_ctrl->is_cont_scanning' flags in p_ctrl.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Resets the digital part of the device (soft reset) or both digital and analog parts (hard)
 *   according to the selected reset type.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning After reset you must rewrite all configuration registers by calling R_RAA489220_Setup!
 * @param[in]       p_ctrl                              Pointer to the control structure
 * @param[in]       type                                Hard or soft reset type selector.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Invalid reset type was selected.
 * @retval          BMS_ERR_...                         Inherit from bfe_reset_soft().
 * @retval          BMS_ERR_...                         Inherit from bfe_reset_hard().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Reset (st_bfe_ctrl_t * const p_ctrl, e_bfe_reset_type_t type)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    switch(type)
    {
        case BFE_RESET_TYPE_SOFT:
        {
            bfe_err = bfe_reset_soft(p_ctrl);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            break;
        }

        case BFE_RESET_TYPE_HARD:
        {
            bfe_err = bfe_reset_hard(p_ctrl);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            break;
        }

        default:
        {
            return BFE_ERR_INVALID_ARGUMENT; // Error return

            break;
        }
    }

    /* Update flags */
    p_ctrl->is_low_power = false;
    p_ctrl->is_cont_scanning = false;

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_Reset
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_ModeSet */
/******************************************************************************************************************//**
 * This function forces the BFE to enter sleep mode or wakes it up. Mode or state is selected with the mode input
 * parameter from a predefined list of modes. It modifies the 'p_ctrl->is_low_power', 'p_ctrl->is_cont_scanning'
 * and  'p_ext_ctrl->weak_regulator' flags in p_ctrl.
 *
 * The supported states and modes are:
 * - BFE_STATE_RESET
 * - BFE_MODE_IDLE
 * - BFE_MODE_AUTOSCAN
 * - BFE_MODE_LOW_POWER_MODE
 * - BFE_MODE_SHIP
 *
 * Keep in mind that when put in Reset State the BFE makes a transition to Idle Mode afterwards.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Changes bits into Device Setup Registers 1, 2 and 3 so that the BFE enters the desired mode.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning After reset you must rewrite all configuration registers by calling R_RAA489220_Setup!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       mode                                Mode selection.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_UNSUPPORTED_MODE            Invalid mode was selected.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_WRITE_VERIFY                Write command verification error.
 * @retval          BMS_ERR_...                         Inherit from bfe_i2c_register_read().
 * @retval          BMS_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BMS_ERR_...                         Inherit from bfe_reset_soft().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_ModeSet (st_bfe_ctrl_t * const p_ctrl, e_bfe_mode_t mode)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    st_raa489220_ext_cfg_t   * p_ext_cfg  = (st_raa489220_ext_cfg_t *) p_ctrl->p_cfg->p_extend;
    st_raa489220_ext_ctrl_t  * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;

    u_raa489220_config1_reg_t system_config_1 = {0};
    u_raa489220_config2_reg_t system_config_2 = {0};
    u_raa489220_config3_reg_t system_config_3 = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Read System Config 1 Register. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_1, &p_bfe_regs->system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    system_config_1.value = p_bfe_regs->system_config_1.value;

    /* Read System Config 2 Register. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    system_config_2.value = p_bfe_regs->system_config_2.value;

    /* Read System Config 3 Register. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_3, &p_bfe_regs->system_config_3.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    system_config_3.value = p_bfe_regs->system_config_3.value;

    system_config_3.value_b.AUTO_SCAN_EN = 0;

    p_ctrl->is_cont_scanning = false;
    p_ctrl->is_low_power = false;
    p_ext_ctrl->weak_regulator = false;

    switch(mode)
    {
        case BFE_STATE_RESET:
        {
            bfe_err = bfe_reset_soft(p_ctrl);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            break;
        }

        case BFE_MODE_IDLE:
        {
            system_config_3.value_b.AUTO_SCAN_EN = 0;

            break;
        }

        case BFE_MODE_AUTOSCAN:
        {
            system_config_2.value_b.LP_REG = (uint16_t) ((p_ext_cfg->lp_mode_settings >> 4) & 0x01);
            system_config_3.value_b.SLEEP_EN = (uint16_t) (p_ext_cfg->lp_mode_settings & 0x01);

            system_config_3.value_b.AUTO_SCAN_EN = 1;

            system_config_3.value_b.SCAN_DELAY = (uint16_t) (p_ext_cfg->scan_delay & 0x03);
            system_config_3.value_b.FAULT_DELAY = (uint16_t) (p_ext_cfg->fault_delay & 0x03);

            p_ctrl->is_cont_scanning = true;

            if(p_ext_cfg->lp_mode_settings == BFE_SHIP_MODE)
            {
                p_ext_ctrl->weak_regulator = true;
            }
            else
            {
                ;
            }

            break;
        }

        case BFE_MODE_LOW_POWER:
        {
            system_config_1.value_b.GO2LPM = 1;

            system_config_2.value_b.LP_REG = 1;
            system_config_3.value_b.SLEEP_EN = 1;

            system_config_3.value_b.AUTO_SCAN_EN = 0;

            p_ctrl->is_low_power = true;

            break;
        }

        case BFE_MODE_SHIP:
        {
            system_config_1.value_b.GO2LPM = 1;

            system_config_2.value_b.LP_REG = 0;
            system_config_3.value_b.SLEEP_EN = 0;

            system_config_3.value_b.AUTO_SCAN_EN = 0;

            p_ctrl->is_low_power = true;
            p_ext_ctrl->weak_regulator = true;

            break;
        }

        default:
        {
            return BFE_ERR_UNSUPPORTED_MODE; // Error return

            break;
        }
    }

    /* Set System Configure 2 Register */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_2, system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(system_config_2.value == p_bfe_regs->system_config_2.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Unlock thresholds registers write */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_LOCK_KEY_0);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_UNLOCK_KEY_1);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_UNLOCK_KEY_2);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_UNLOCK_KEY_3);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Set System Configure 3 Register */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_3, system_config_3.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Lock thresholds registers write */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->user_sel_mode, BFE_USER_SELECT_LOCK_KEY_0);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_3, &p_bfe_regs->system_config_3.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(system_config_3.value == p_bfe_regs->system_config_3.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    if(p_ctrl->is_low_power == true)
    {
        /* Set System Configure 1 Register */
        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_1, system_config_1.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.
    }
    else
    {
        ;
    }

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_ModeSet
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_ModeRead */
/******************************************************************************************************************//**
 * This function gets the current BFE mode.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Compare control structure parameters to obtain the current mode.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_mode                              Pointer to the obtained mode.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_ModeRead (st_bfe_ctrl_t * const p_ctrl, e_bfe_mode_t * const p_mode)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_ext_ctrl_t * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_mode, BFE_ERR_INVALID_POINTER);

#endif // BFE_CFG_PARAM_CHECKING_EN

    if(p_ctrl->is_low_power == true)
    {
        if(p_ext_ctrl->weak_regulator == true)
        {
            *p_mode = BFE_MODE_SHIP;
        }
        else
        {
            *p_mode = BFE_MODE_LOW_POWER;
        }
    }
    else if (p_ctrl->is_cont_scanning == true)
    {
        *p_mode = BFE_MODE_AUTOSCAN;
    }
    else
    {
        *p_mode = BFE_MODE_IDLE;
    }

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_ModeRead
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_CommTest */
/******************************************************************************************************************//**
 * This function tests the communication between the MCU and the BFE.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Read Device ID Register to test communication.
 * - Compare the acquired Device ID value.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_COMM_FAULT                  Communication fault was found.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_CommTest (st_bfe_ctrl_t * const p_ctrl)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    st_raa489220_info_t      * p_bfe_info = (st_raa489220_info_t *) p_ctrl->p_bfe_info;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Read Device ID Register */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->device_id, &p_bfe_regs->device_id.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, BFE_ERR_COMM_FAULT); // Check for errors.

    /* Compare Device ID to verify correct register read. */
    BFE_ERROR_RETURN(p_bfe_info->device_id == p_bfe_regs->device_id.value, BFE_ERR_COMM_FAULT);

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_CommTest
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_SelfDiag */
/******************************************************************************************************************//**
 * This function runs a self-diagnostic test for the BFE.
 * The only supported option is 'BFE_FULL_TEST'!
 *
 *  If the BFE is in Auto Scan Mode and a fault is detected 'is_cont_scanning' and 'is_low_power' flags are updated.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Triggers a Single System Scan that includes regulators test, cells and temperature inputs open-wire test,
 * oscillator test and other (Please, refer to the RAA489220 datasheet).
 * - Waits for the scan to complete.
 * - Reads the Fault Register to check for any asserted flags.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Always check the 'is_fault_detected' flag in p_ctrl after calling this function! Keep in mind that
 * open wire test behavior depends on the 'p_ext_cfg->open_wire_test_enable' setting!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       option                              Selected option for the self-diagnostic
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Invalid option was selected.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BFE_ERR_...                         Inherit from bfe_meas_complete_wait().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_SelfDiag (st_bfe_ctrl_t * const p_ctrl, e_bfe_diag_option_t option)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs  = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    u_raa489220_meas_sel_reg_t measure_select = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    switch(option)
    {
        case BFE_FULL_TEST:
        {
            /* Select and trigger single system scan. */
            measure_select.value = 0;
            measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
            measure_select.value_b.MEAS_SELECT = BFE_MEAS_SINLE_SYS_SCAN;

            bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            /* Wait until measurement has completed. */
            bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            /* Read Fault Register */
            bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->faults, &p_bfe_regs->faults.value);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            if(p_bfe_regs->faults.value == 0)
            {
                p_ctrl->is_fault_detected = false;
            }
            else
            {
                p_ctrl->is_fault_detected = true;
            }

            /* Check if the BFE has automatically switched to Low Power Modes. */
            if((p_ctrl->is_fault_detected == true) && (p_ctrl->is_cont_scanning == true))
            {
                p_ctrl->is_cont_scanning = false;
                p_ctrl->is_low_power = true;
            }
            else
            {
                p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.
            }

            break;
        }

        default:
        {
            bfe_err = BFE_ERR_INVALID_ARGUMENT; // Error return

            break;
        }
    }

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_SelfDiag
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_MemCheck */
/******************************************************************************************************************//**
 * This function checks if the default configuration registers values are loaded correctly.
 * The oly supported option is 'BFE_CHECK_DEF_VALS'!
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Reads the current configuration registers values.
 * - Compares the obtained values with the expected defaults.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       option                              Selected option for the memory test
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Invalid option was selected.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_MemCheck (st_bfe_ctrl_t * const p_ctrl, e_bfe_mem_check_option_t option)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

        switch(option)
        {
            case BFE_CHECK_DEF_VALS:
            {
                /* Read all configuration registers */
                bfe_err = bfe_i2c_register_read(p_ctrl,
                                                p_bfe_regs->system_config_1,
                                                &p_bfe_regs->system_config_1.value);
                BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

                bfe_err = bfe_i2c_register_read(p_ctrl,
                                                p_bfe_regs->system_config_2,
                                                &p_bfe_regs->system_config_2.value);
                BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

                bfe_err = bfe_i2c_register_read(p_ctrl,
                                                p_bfe_regs->system_config_3,
                                                &p_bfe_regs->system_config_3.value);
                BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

                bfe_err = bfe_i2c_register_read(p_ctrl,
                                                p_bfe_regs->ov_eoc,
                                                &p_bfe_regs->ov_eoc.value);
                BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

                bfe_err = bfe_i2c_register_read(p_ctrl,
                                                p_bfe_regs->dvcov_uv,
                                                &p_bfe_regs->dvcov_uv.value);
                BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

                bfe_err = bfe_i2c_register_read(p_ctrl,
                                                p_bfe_regs->scc_ot_ut,
                                                &p_bfe_regs->scc_ot_ut.value);
                BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

                bfe_err = bfe_i2c_register_read(p_ctrl,
                                                p_bfe_regs->current,
                                                &p_bfe_regs->current.value);
                BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

                /* Compare register defaults with read values */
                BFE_ERROR_RETURN((p_bfe_regs->system_config_1.value & 0x00FF)
                              == (p_bfe_regs->system_config_1.def_value & 0x00FF),
                                 BFE_ERR_REGISTER_RESET_UNMATCHED);
                BFE_ERROR_RETURN(p_bfe_regs->system_config_2.value == p_bfe_regs->system_config_2.def_value,
                                 BFE_ERR_REGISTER_RESET_UNMATCHED);
                BFE_ERROR_RETURN(p_bfe_regs->system_config_3.value == p_bfe_regs->system_config_3.def_value,
                                 BFE_ERR_WRITE_VERIFY);
                BFE_ERROR_RETURN(p_bfe_regs->ov_eoc.value == p_bfe_regs->ov_eoc.def_value,
                                 BFE_ERR_REGISTER_RESET_UNMATCHED);
                BFE_ERROR_RETURN(p_bfe_regs->dvcov_uv.value == p_bfe_regs->dvcov_uv.def_value,
                                 BFE_ERR_REGISTER_RESET_UNMATCHED);
                BFE_ERROR_RETURN(p_bfe_regs->scc_ot_ut.value == p_bfe_regs->scc_ot_ut.def_value,
                                 BFE_ERR_REGISTER_RESET_UNMATCHED);
                BFE_ERROR_RETURN(p_bfe_regs->current.value == p_bfe_regs->current.def_value,
                                 BFE_ERR_REGISTER_RESET_UNMATCHED);

                p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

                break;
            }

            default:
            {
                bfe_err = BFE_ERR_INVALID_ARGUMENT; // Error return

                break;
            }
        }

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_MemCheck
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_VPackGet */
/******************************************************************************************************************//**
 * This function acquires the battery pack voltage. You must always select to trigger a measurement as the function
 * does not support reading of prior measured values. The return data is converted into voltage.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Select a measurement type.
 * - Trigger a measurement.
 * - Wait for the Busy bit to clear.
 * - Read and convert the ADC value.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Always set 'trigger' input parameter to be 'true'! Keep in mind the measured values are not compared
 * for faults in the BFE internally.
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_value                             Pointer to the acquired data.
 * @param[in]       trigger                             Triggered a measurement or only read data for the last one.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Input argument is invalid.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 * @retval          BFE_ERR_...                         Inherit from bfe_meas_complete_wait().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_VPackGet (st_bfe_ctrl_t * const p_ctrl,
                                  float         * const p_value,
                                  bool                  trigger)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;

    u_raa489220_meas_sel_reg_t measure_select = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,  BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_value, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

    /* Verify trigger is set. */
    BFE_ERROR_RETURN(true == trigger, BFE_ERR_INVALID_ARGUMENT);

#else

    FSP_PARAMETER_NOT_USED(trigger); // Mute unused parameter

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Select and trigger pack voltage measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_VPACK;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy measured pack voltage */
    *p_value = bfe_adc_to_vpack(p_bfe_regs->thermistor_2.value);

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_VPackGet
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_IPackGet */
/******************************************************************************************************************//**
 * This function acquires the battery pack current. You must always select to trigger a measurement as the function
 * does not support reading of prior measured values. The function measures both charge and discharge currents.
 * The return data is converted into currents.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Select a measurement type.
 * - Trigger a measurement.
 * - Wait for the Busy bit to clear.
 * - Read and convert the ADC values.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Always set 'trigger' input parameter to be 'true'! Keep in mind the measured values are not compared
 * for faults in the BFE internally.
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_values                            Pointer to the acquired data structure.
 * @param[in]       trigger                             Triggered a measurement or only read data for the last one.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Input argument is invalid.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 * @retval          BFE_ERR_...                         Inherit from bfe_meas_complete_wait().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_IPackGet (st_bfe_ctrl_t        * const p_ctrl,
                                  st_bfe_i_pack_meas_t * const p_values,
                                  bool                         trigger)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    st_raa489220_ext_cfg_t   * p_ext_cfg  = (st_raa489220_ext_cfg_t *) p_ctrl->p_cfg->p_extend;

    u_raa489220_meas_sel_reg_t measure_select = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,  BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_values, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

    /* Verify trigger is set. */
    BFE_ERROR_RETURN(true == trigger, BFE_ERR_INVALID_ARGUMENT);

#else

    FSP_PARAMETER_NOT_USED(p_ctrl); // Mute unused parameter
    FSP_PARAMETER_NOT_USED(trigger); // Mute unused parameter

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Select and trigger discharge current measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_IPACK_DISCH;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy discharge current */
    p_values->i_discharge = bfe_adc_to_ipack(p_bfe_regs->thermistor_2.value, p_ext_cfg->r_shunt_discharge_mohm);

    /* Select and trigger discharge current measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_IPACK_CH;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy charge current */
    p_values->i_charge = bfe_adc_to_ipack(p_bfe_regs->thermistor_2.value, p_ext_cfg->r_shunt_charge_mohm);

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_IPackGet
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_VCellsGet */
/******************************************************************************************************************//**
 * This function acquires the voltages of all cells in the stack and the battery pack voltage. You must always select
 * to trigger a measurement as the function does not support reading of prior measured values. The return data
 * is converted into voltages.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Select a measurement type.
 * - Trigger a measurement.
 * - Wait for the Busy bit to clear.
 * - Read and convert the ADC values.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Always set 'trigger' input parameter to be 'true'! Keep in mind the measured values are not compared
 * for faults in the BFE internally.
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_values                            Pointer to the acquired data structure.
 * @param[in]       trigger                             Triggered a measurement or only read data for the last one.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Input argument is invalid.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 * @retval          BFE_ERR_...                         Inherit from bfe_meas_complete_wait().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_VCellsGet (st_bfe_ctrl_t    * const p_ctrl,
                                   bfe_vcell_meas_t * const p_values,
                                   bool                     trigger)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t  * p_bfe_regs  = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    u_raa489220_vcells_meas_t * p_meas_data = (u_raa489220_vcells_meas_t *) p_values;

    u_raa489220_meas_sel_reg_t measure_select = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,   BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_values, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

    /* Verify trigger is set. */
    BFE_ERROR_RETURN(true == trigger, BFE_ERR_INVALID_ARGUMENT);

#else

    FSP_PARAMETER_NOT_USED(trigger); // Mute unused parameter

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Select and trigger pack voltage measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_VPACK;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy measured pack voltage */
    p_meas_data->measurements.v_pack = bfe_adc_to_vpack(p_bfe_regs->thermistor_2.value);

    for(uint16_t i = 0; i < BFE_CELL_NUM; i++ )
    {
        /* Select and trigger cell voltage measurement. */
        measure_select.value = 0;
        measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
        measure_select.value_b.MEAS_SELECT = (e_raa489220_meas_select_t) (BFE_MEAS_VOLT_CELL_1 + i);

        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Wait until measurement has completed. */
        bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Read measured data. */
        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Convert and copy cell voltage */
        p_meas_data->vector[i] = bfe_adc_to_vcell(p_bfe_regs->thermistor_2.value);
    }

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_VCellsGet
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_Temps */
/******************************************************************************************************************//**
 * This function acquires the external temperature voltage. You must always select to trigger a measurement as
 * the function does not support reading of prior measured values. The return data is converted into voltages.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Enable VTEMP pin voltage.
 * - Select a measurement type.
 * - Trigger a measurement.
 * - Wait for the Busy bit to clear.
 * - Read and convert the ADC values.
 * - Disable VTEMP pin voltage.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Always set 'trigger' input parameter to be 'true'! Keep in mind the measured values are not compared
 * for faults in the BFE internally.
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_values                            Pointer to the acquired data structure.
 * @param[in]       trigger                             Triggered a measurement or only read data for the last one.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Input argument is invalid.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_INVALID_CONF                The thermistor configuration selection is invalid.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 * @retval          BFE_ERR_...                         Inherit from bfe_meas_complete_wait().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_Temps (st_bfe_ctrl_t   * const p_ctrl,
                               bfe_temp_meas_t * const p_values,
                               bool                    trigger)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs  = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    uint16_t                 * p_temps_cfg = (uint16_t *) p_ctrl->p_cfg->p_temps_select;
    u_raa489220_temps_meas_t * p_meas_data = (u_raa489220_temps_meas_t *) p_values;

    u_raa489220_meas_sel_reg_t measure_select  = {0};
    u_raa489220_config2_reg_t  system_config_2 = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,   BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_values, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

    /* Verify trigger is set. */
    BFE_ERROR_RETURN(true == trigger, BFE_ERR_INVALID_ARGUMENT);

#else

    FSP_PARAMETER_NOT_USED(trigger); // Mute unused parameter

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Check if temperatures are monitored. */
    BFE_ERROR_RETURN(BFE_THERM_OFF != *p_temps_cfg, BFE_ERR_INVALID_CONF);

    /* Enable VTEMP pin voltage */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    system_config_2.value = p_bfe_regs->system_config_2.value;
    system_config_2.value_b.VTEMP_EN = BFE_ENABLE_VTEMP;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_2, system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(system_config_2.value == p_bfe_regs->system_config_2.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    /* Select and trigger VTEMP pin voltage measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_VTEMP;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read therm1. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy vtemp pin voltage measurement. */
    p_meas_data->measurements.v_temp = bfe_adc_to_vregpin(p_bfe_regs->thermistor_2.value);

    /* Select and trigger thermistor 1 measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_THERM_1;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read therm1. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy thermistor 1. */
    p_meas_data->measurements.v_therm_1 = bfe_adc_to_vtherm(p_bfe_regs->thermistor_2.value);

    if(*p_temps_cfg == BFE_THERM_1_2)
    {
        /* Select and trigger thermistor 2 measurement. */
        measure_select.value = 0;
        measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
        measure_select.value_b.MEAS_SELECT = BFE_MEAS_THERM_2;

        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Wait until measurement has completed. */
        bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Read therm2. */
        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Convert and copy thermistor 2. */
        p_meas_data->measurements.v_therm_2 = bfe_adc_to_vtherm(p_bfe_regs->thermistor_2.value);
    }
    else
    {
        p_meas_data->measurements.v_therm_2 = 0;
    }

    /* Disable VTEMP pin voltage */
    system_config_2.value_b.VTEMP_EN = BFE_DISABLE_VTEMP;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_2, system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    BFE_ERROR_RETURN(system_config_2.value == p_bfe_regs->system_config_2.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_Temps
 *********************************************************************************************************************/


/* Function Name: R_RAA489220_AllGet */
/******************************************************************************************************************//**
 * This function acquires the voltages of all cells,the battery pack voltage, charge and discharge currents,
 * external temperatures and regulator voltage. You must always select to trigger a measurement as the function
 * does not support reading of prior measured values. The return data is converted into voltages and currents.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Select a measurement type.
 * - Trigger a measurement.
 * - Wait for the Busy bit to clear.
 * - Read and convert the ADC values.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Always set 'trigger' input parameter to be 'true'! Keep in mind the measured values are not compared
 * for faults in the BFE internally.
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_values                            Pointer to the acquired data structure.
 * @param[in]       trigger                             Triggered a measurement or only read data for the last one.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Input argument is invalid.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 * @retval          BFE_ERR_...                         Inherit from bfe_meas_complete_wait().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_AllGet (st_bfe_ctrl_t  * const p_ctrl,
                                bfe_all_meas_t * const p_values,
                                bool                   trigger)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs  = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    uint16_t                 * p_temps_cfg = (uint16_t *) p_ctrl->p_cfg->p_temps_select;
    st_raa489220_ext_cfg_t   * p_ext_cfg   = (st_raa489220_ext_cfg_t *) p_ctrl->p_cfg->p_extend;
    u_raa489220_all_meas_t   * p_meas_data = (u_raa489220_all_meas_t *) p_values;

    u_raa489220_meas_sel_reg_t measure_select = {0};
    u_raa489220_config2_reg_t  system_config_2 = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,   BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_values, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

    /* Verify trigger is set. */
    BFE_ERROR_RETURN(true == trigger, BFE_ERR_INVALID_ARGUMENT);

#else

    FSP_PARAMETER_NOT_USED(trigger); // Mute unused parameter

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Select and trigger pack voltage measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_VPACK;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy measured pack voltage */
    p_meas_data->measurements.v_pack = bfe_adc_to_vpack(p_bfe_regs->thermistor_2.value);

    for(uint16_t i = 0; i < BFE_CELL_NUM; i++ )
    {
        /* Select and trigger cell voltage measurement. */
        measure_select.value = 0;
        measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
        measure_select.value_b.MEAS_SELECT = (e_raa489220_meas_select_t) (BFE_MEAS_VOLT_CELL_1 + i);

        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Wait until measurement has completed. */
        bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Read measured data. */
        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Convert and copy cell voltage */
        p_meas_data->vector[i] = bfe_adc_to_vcell(p_bfe_regs->thermistor_2.value);
    }

    /* Select and trigger pin V3P3 voltage measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_V3P3;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy measured pack voltage */
    p_meas_data->measurements.v_pin_v3p3 = bfe_adc_to_vregpin(p_bfe_regs->thermistor_2.value);

    /* Select and trigger pin V2P5 voltage measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_V2P5;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy measured pack voltage */
    p_meas_data->measurements.v_pin_v2p5 = bfe_adc_to_vregpin(p_bfe_regs->thermistor_2.value);

    /* Select and trigger second  Band Gap reference voltage measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_VBG2;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy measured pack voltage */
    p_meas_data->measurements.v_ref_bg2 = bfe_adc_to_vbg2ref(p_bfe_regs->thermistor_2.value);

    /* Select and trigger discharge current measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_IPACK_DISCH;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy discharge current */
    p_meas_data->measurements.i_discharge = bfe_adc_to_ipack(p_bfe_regs->thermistor_2.value,
                                                             p_ext_cfg->r_shunt_discharge_mohm);

    /* Select and trigger discharge current measurement. */
    measure_select.value = 0;
    measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
    measure_select.value_b.MEAS_SELECT = BFE_MEAS_IPACK_CH;

    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait until measurement has completed. */
    bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read measured data. */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Convert and copy charge current */
    p_meas_data->measurements.i_charge = bfe_adc_to_ipack(p_bfe_regs->thermistor_2.value,
                                                          p_ext_cfg->r_shunt_charge_mohm);

    if(*p_temps_cfg != BFE_THERM_OFF)
    {
        /* Enable VTEMP pin voltage */
        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        system_config_2.value = p_bfe_regs->system_config_2.value;
        system_config_2.value_b.VTEMP_EN = BFE_ENABLE_VTEMP;

        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_2, system_config_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        BFE_ERROR_RETURN(system_config_2.value == p_bfe_regs->system_config_2.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

        /* Select and trigger VTEMP pin voltage measurement. */
        measure_select.value = 0;
        measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
        measure_select.value_b.MEAS_SELECT = BFE_MEAS_VTEMP;

        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Wait until measurement has completed. */
        bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Read therm1. */
        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Convert and copy vtemp pin voltage measurement. */
        p_meas_data->measurements.v_temp = bfe_adc_to_vregpin(p_bfe_regs->thermistor_2.value);

        /* Select and trigger thermistor 1 measurement. */
        measure_select.value = 0;
        measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
        measure_select.value_b.MEAS_SELECT = BFE_MEAS_THERM_1;

        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Wait until measurement has completed. */
        bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Read therm1. */
        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Convert and copy thermistor 1. */
        p_meas_data->measurements.v_therm_1 = bfe_adc_to_vtherm(p_bfe_regs->thermistor_2.value);

        if(*p_temps_cfg == BFE_THERM_1_2)
        {
            /* Select and trigger thermistor 2 measurement. */
            measure_select.value = 0;
            measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
            measure_select.value_b.MEAS_SELECT = BFE_MEAS_THERM_2;

            bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            /* Wait until measurement has completed. */
            bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            /* Read therm2. */
            bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->thermistor_2, &p_bfe_regs->thermistor_2.value);
            BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

            /* Convert and copy thermistor 2. */
            p_meas_data->measurements.v_therm_2 = bfe_adc_to_vtherm(p_bfe_regs->thermistor_2.value);
        }
        else
        {
            p_meas_data->measurements.v_therm_2 = 0;
        }

        /* Disable VTEMP pin voltage */
        system_config_2.value_b.VTEMP_EN = BFE_DISABLE_VTEMP;

        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_2, system_config_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_2, &p_bfe_regs->system_config_2.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        BFE_ERROR_RETURN(system_config_2.value == p_bfe_regs->system_config_2.value, BFE_ERR_WRITE_VERIFY);

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    }
    else
    {
        p_meas_data->measurements.v_therm_1 = 0;
    }

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_AllGet
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_SysScanGet */
/******************************************************************************************************************//**
 * This function runs a single system scan and reads the acquired values for pack voltage, min and max cell voltage,
 * charge and discharge current and temperatures. You can select whether to trigger a measurement and read the obtained
 * values or only read the last obtained values. The return data is converted into voltages and currents.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Select a measurement type.
 * - Trigger a measurement.
 * - Wait for the Busy bit to clear.
 * - Read and convert the ADC values.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Keep in mind the measured values are not compared for faults in the BFE internally.
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_values                            Pointer to the acquired data structure.
 * @param[in]       trigger                             Triggered a measurement or only read data for the last one.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_INVALID_ARGUMENT            Input argument is invalid.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_block_read().
 * @retval          BFE_ERR_...                         Inherit from bfe_meas_complete_wait().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_SysScanGet (st_bfe_ctrl_t    * const p_ctrl,
                                    bfe_other_meas_t * const p_values,
                                    bool                     trigger)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t    * p_bfe_regs  = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    uint16_t                    * p_temps_cfg = (uint16_t *) p_ctrl->p_cfg->p_temps_select;
    st_raa489220_ext_cfg_t      * p_ext_cfg   = (st_raa489220_ext_cfg_t *) p_ctrl->p_cfg->p_extend;
    u_raa489220_sys_scan_meas_t * p_meas_data = (u_raa489220_sys_scan_meas_t *) p_values;

    u_raa489220_meas_sel_reg_t measure_select = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,   BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_values, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Trigger single system scan, if selected. */
    if(trigger == true)
    {
        /* Select and trigger measurement. */
        measure_select.value = 0;
        measure_select.value_b.TRIGGER_MEAS = BFE_SET_TRIGGER;
        measure_select.value_b.MEAS_SELECT = BFE_MEAS_SINLE_SYS_SCAN;

        bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->measure_select, measure_select.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        /* Wait until measurement has completed. */
        bfe_err = bfe_meas_complete_wait(p_ctrl, measure_select.value_b.MEAS_SELECT);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.
    }
    else
    {
        ; // Do nothing.
    }

    /* Read measured data. */
    bfe_err = bfe_i2c_block_read(p_ctrl, p_bfe_regs->cmnd_measurements, &s_data_buffer[0]);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    p_bfe_regs->pack_voltage.value = s_data_buffer[0];
    p_bfe_regs->cell_min_voltage.value = s_data_buffer[1];
    p_bfe_regs->cell_max_voltage.value = s_data_buffer[2];
    p_bfe_regs->thermistor_1.value = s_data_buffer[3];
    p_bfe_regs->thermistor_2.value = s_data_buffer[4];
    p_bfe_regs->discharge_current.value = s_data_buffer[5];
    p_bfe_regs->charge_current.value = s_data_buffer[6];

    /* Convert and copy measured voltages */
    p_meas_data->measurements.v_pack     = bfe_adc_to_vpack(p_bfe_regs->pack_voltage.value);
    p_meas_data->measurements.v_cell_max = bfe_adc_to_vcell(p_bfe_regs->cell_min_voltage.value);
    p_meas_data->measurements.v_cell_min = bfe_adc_to_vcell(p_bfe_regs->cell_max_voltage.value);

    /* Convert and copy thermistor 1. */
    if(*p_temps_cfg != BFE_THERM_OFF)
    {
        p_meas_data->measurements.v_therm_1 = bfe_adc_to_vtherm(p_bfe_regs->thermistor_1.value);
    }
    else
    {
        p_meas_data->measurements.v_therm_1 = 0;
    }

    /* Convert and copy thermistor 2. */
    if(*p_temps_cfg == BFE_THERM_1_2)
    {
        p_meas_data->measurements.v_therm_2 = bfe_adc_to_vtherm(p_bfe_regs->thermistor_2.value);
    }
    else
    {
        p_meas_data->measurements.v_therm_2 = 0;
    }

    /* Convert and copy measured currents */
    p_meas_data->measurements.i_discharge = bfe_adc_to_ipack(p_bfe_regs->discharge_current.value,
                                                             p_ext_cfg->r_shunt_discharge_mohm);
    p_meas_data->measurements.i_charge    = bfe_adc_to_ipack(p_bfe_regs->charge_current.value,
                                                             p_ext_cfg->r_shunt_charge_mohm);

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_SysScanGet
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_FaultsAllRead */
/******************************************************************************************************************//**
 * This function reads the fault registers. It modifies the 'p_ctrl->is_low_power' flags in p_ctrl.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Reads Fault Register.
 * - Copies fault bits to faults data structure.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_faults                            Pointer to faults data structure.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_FaultsAllRead (st_bfe_ctrl_t * const p_ctrl, bfe_faults_t * const p_faults)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    st_raa489220_faults_t    * p_bfe_faults = (st_raa489220_faults_t *) p_faults;

    u_raa489220_fault_reg_t fault_register = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,   BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_faults, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Read Fault Register */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->faults, &p_bfe_regs->faults.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    fault_register.value = p_bfe_regs->faults.value;

    /* Check for charge overcurrent fault. */
    p_bfe_faults->flt_charge_over_current = (fault_register.value_b.COCF > 0) ? true : false;

    /* Check for discharge overcurrent fault. */
    p_bfe_faults->flt_discharge_over_current = (fault_register.value_b.DOCF > 0) ? true : false;

    /* Check for short-circuit fault. */
    p_bfe_faults->flt_short_circuit = (fault_register.value_b.SCF > 0) ? true : false;

    /* Check for under-temperature fault. */
    p_bfe_faults->flt_under_temeprature = (fault_register.value_b.UTF > 0) ? true : false;

    /* Check for over-temperature fault. */
    p_bfe_faults->flt_over_temeprature = (fault_register.value_b.OTF > 0) ? true : false;

    /* Check for undervoltage fault. */
    p_bfe_faults->flt_undervoltage = (fault_register.value_b.UVF > 0) ? true : false;

    /* Check for overvoltage fault. */
    p_bfe_faults->flt_overvoltage = (fault_register.value_b.OVF > 0) ? true : false;

    /* Check for delta cell voltage fault. */
    p_bfe_faults->flt_delta_cell_ov = (fault_register.value_b.DCVF > 0) ? true : false;

    /* Check for communication time out. */
    p_bfe_faults->flt_comm_timeout = (fault_register.value_b.COMMTO > 0) ? true : false;

    /* Check for self-test fault. */
    p_bfe_faults->flt_self_test = (fault_register.value_b.STF > 0) ? true : false;

    /* Check for lock out fault. */
    p_bfe_faults->flt_lock_out = (fault_register.value_b.LOF > 0) ? true : false;

    /* Check for VTEMP fault. */
    p_bfe_faults->flt_vtemp = (fault_register.value_b.VTMPF > 0) ? true : false;

    /* Check for internal over-temperature fault. */
    p_bfe_faults->flt_internal_over_temp = (fault_register.value_b.IOTF > 0) ? true : false;

    /* Check for open-wire fault. */
    p_bfe_faults->flt_open_wire = (fault_register.value_b.OWF > 0) ? true : false;

    /* Check for REG Fault V3P3 and V2P5 pins fault. */
    p_bfe_faults->flt_regulator = (fault_register.value_b.REGF > 0) ? true : false;

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_FaultsAllRead
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_FaultsCheck */
/******************************************************************************************************************//**
 * This function checks ALERT pin and reads the Fault Register.
 * When a fault is detected the 'is_fault_detected' flag is set in p_ctrl. If the BFE is in auto scan mode and a
 * fault is detected 'is_cont_scanning' and 'is_low_power' flags are updated.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Checks the ALERT pin for assertion if its normal functionality is used.
 * - Check Fault Register for set flags.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Always check the 'is_fault_detected' flag in p_ctrl after calling this function!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_FaultsCheck (st_bfe_ctrl_t * const p_ctrl)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    st_raa489220_ext_cfg_t   * p_ext_cfg  = (st_raa489220_ext_cfg_t *) p_ctrl->p_cfg->p_extend;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,  BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    if(p_ext_cfg->alert_pin_cfg == BFE_ALRT_NORMAL)
    {
        /* Check ALERT pin. */
        if(s_alert_event_flag == true)
        {
            s_alert_event_flag = false;

            p_ctrl->is_fault_detected = true;
        }
        else
        {
            p_ctrl->is_fault_detected = false;
        }
    }
    else if(p_ctrl->is_low_power == false) // Check for low power mode
    {
        /* Read Fault Register */
        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->faults, &p_bfe_regs->faults.value);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

        if(p_bfe_regs->faults.value == 0)
        {
            p_ctrl->is_fault_detected = false;
        }
        else
        {
            p_ctrl->is_fault_detected = true;
        }
    }
    else
    {
        ;
    }

    /* Check if the BFE has automatically switched to Low Power Modes. */
    if((p_ctrl->is_fault_detected == true) && (p_ctrl->is_cont_scanning == true))
    {
        p_ctrl->is_cont_scanning = false;
        p_ctrl->is_low_power = true;
    }
    else
    {
        ;
    }

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_FaultsCheck
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_FaultsAllClear */
/******************************************************************************************************************//**
 * This function attempts to clear all faults in the BFE. When all faults are cleared the 'p_success flag' is set.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Sets Clear All Faults bit in System Config 1 Register.
 * - Reads Fault Register to verify fault clear.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Always check the 'p_success' flag in p_ctrl after calling this function!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[out]      p_success                           Pointer to boolean faults clear status variable.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_WRITE_VERIFY                Write command verification error.
 * @retval          BFE_ERR_...                         Inherit from bfe_spi_msg_send_resp_get().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_FaultsAllClear (st_bfe_ctrl_t * const p_ctrl, bool * const p_success)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;

    u_raa489220_config1_reg_t    system_config_1 = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,    BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_success, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Read System Config 1 Register */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_1, &p_bfe_regs->system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Set the Clear All Faults Bit */
    system_config_1.value = p_bfe_regs->system_config_1.value;
    system_config_1.value_b.CLR_ALL_FAULTS = 1;

    /* Write System Config 1 Register to clear all faults. */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_1, system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Read Fault Register */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->faults, &p_bfe_regs->faults.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Check for any fault flags left */
    if(p_bfe_regs->faults.value == 0)
    {
        *p_success = true;
        p_ctrl->is_fault_detected = false;
    }
    else
    {
        *p_success = false;
    }

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_FaultsAllClear
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_CellBalanceCtrl */
/******************************************************************************************************************//**
 * This function is unsupported by the current API implementation.
 *
 * @warning This function is not supported by the current API implementation!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       p_bal_cfg                           Pointer to the balancing configuration structure.
 * @param[in]       ctrl_option                         Specify action to enable or inhibit cell balancing.
 * @retval          BFE_ERR_UNSUPPORTED_FEATURE         This feature is unsupported by the current API implementation.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_CellBalanceCtrl (st_bfe_ctrl_t       * const p_ctrl,
                                         bfe_cb_cfg_t        * const p_bal_cfg,
                                         e_bfe_process_ctrl_t        ctrl_option)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    FSP_PARAMETER_NOT_USED(p_ctrl); // Mute unused parameter
    FSP_PARAMETER_NOT_USED(p_bal_cfg); // Mute unused parameter
    FSP_PARAMETER_NOT_USED(ctrl_option); // Mute unused parameter

    /* This FET control is unsupported! */
    bfe_err = BFE_ERR_UNSUPPORTED_FEATURE;

    bfe_crc8_calculate(NULL, 0, NULL); // Mute unused function.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_CellBalanceCtrl
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_IsCellBalancing */
/******************************************************************************************************************//**
 * This function is unsupported by the current API implementation.
 *
 * @warning This function is not supported by the current API implementation!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @retval          BFE_ERR_UNSUPPORTED_FEATURE         This feature is unsupported by the current API implementation.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_IsCellBalancing (st_bfe_ctrl_t * const p_ctrl)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,   BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    FSP_PARAMETER_NOT_USED(p_ctrl); // Mute unused parameter

    /* This FET control is unsupported! */
    bfe_err = BFE_ERR_UNSUPPORTED_FEATURE;

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_IsCellBalancing
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_ContScanCtrl */
/******************************************************************************************************************//**
 * This function is unsupported by the current API implementation.
 *
 * @warning This function is not supported by the current API implementation!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       p_bal_cfg                           Pointer to the continuous scan configuration structure.
 * @param[in]       ctrl_option                         Specify action to enable or inhibit continuous scan.
 * @retval          BFE_ERR_UNSUPPORTED_FEATURE         This feature is unsupported by the current API implementation.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_ContScanCtrl (st_bfe_ctrl_t       * const p_ctrl,
                                      bfe_scan_cont_cfg_t * const p_scan_cfg,
                                      e_bfe_process_ctrl_t        ctrl_option)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,     BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_scan_cfg, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    FSP_PARAMETER_NOT_USED(p_ctrl); // Mute unused parameter
    FSP_PARAMETER_NOT_USED(p_scan_cfg); // Mute unused parameter
    FSP_PARAMETER_NOT_USED(ctrl_option); // Mute unused parameter

    /* This FET control is unsupported! */
    bfe_err = BFE_ERR_UNSUPPORTED_FEATURE;

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_ContScanCtrl
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_WatchdogCtrl */
/******************************************************************************************************************//**
 * This function is unsupported by the current API implementation.
 *
 * @warning This function is not supported by the current API implementation!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       p_options                   Pointer to control options structure.
 * @retval          BFE_ERR_UNSUPPORTED_FEATURE         This feature is unsupported by the current API implementation.
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_WatchdogCtrl (st_bfe_ctrl_t * const p_ctrl, bfe_watchdog_ctrl_t * const p_options)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,            BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_options, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    FSP_PARAMETER_NOT_USED(p_ctrl); // Mute unused parameter
    FSP_PARAMETER_NOT_USED(p_options); // Mute unused parameter

    /* This FET control is unsupported! */
    bfe_err = BFE_ERR_UNSUPPORTED_FEATURE;

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_WatchdogCtrl
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_FETsCtrl */
/******************************************************************************************************************//**
 * This function controls the power FETs. The FET driver output state is set with the parameters 'c_fet_state' and
 * 'd_fet_state'. You can control more than one pair of charge and discharge FETs by using the parameter 'group_num'.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Reads System Config 1 Register.
 * - Changes LCFET and LDFET bits according to input parameters.
 * - Writes back to System Config 1 Register.
 * - Reads System Config 1 Register to verify status of the FETs control bits.
 *
 * @pre The BFE interface should have already been initialized!
 * @warning Call the function only when the device is in Idle Mode!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       group_num                           Select FET group to control.
 * @param[in]       c_fet_state                         Specify state of the charge FET control pin.
 * @param[in]       d_fet_state                         Specify state of the discharge FET control pin.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_INVALID_ARGUMENT            The selected FET state was not correct.
 * @retval          BFE_ERR_FET_CONTROL                 The FET state was not changed according to input parameters.
 * @retval          BFE_ERR_INVALID_OPERATION           The FETs cannot be controlled manually only in Idle Mode.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_FETsCtrl (st_bfe_ctrl_t   * const p_ctrl,
                                  uint8_t                 group_num,
                                  e_bfe_cfet_state_t      c_fet_state,
                                  e_bfe_dfet_state_t      d_fet_state)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;

    u_raa489220_config1_reg_t    system_config_1_tmp1 = {0};
    u_raa489220_config1_reg_t    system_config_1_tmp2 = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    BFE_ERROR_RETURN(p_ctrl->is_cont_scanning == false, BFE_ERR_INVALID_OPERATION);
    BFE_ERROR_RETURN(p_ctrl->is_low_power == false, BFE_ERR_INVALID_OPERATION);

    FSP_PARAMETER_NOT_USED(group_num);

    /* Read System Config 1 Register */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_1, &p_bfe_regs->system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Change the LDFET and CHFET Enable bits. */
    system_config_1_tmp1.value = p_bfe_regs->system_config_1.value;

    /* Copy charge FET state. */
    switch(c_fet_state)
    {
        case BFE_CFET_ON:
        {
            system_config_1_tmp1.value_b.LCFET_EN = 1;

            break;
        }

        case BFE_CFET_OFF:
        {
            system_config_1_tmp1.value_b.LCFET_EN = 0;

            break;
        }

        default:
        {
            return BFE_ERR_INVALID_ARGUMENT; // Error return

            break;
        }
    }

    /* Copy discharge FET state. */
    switch(d_fet_state)
    {
        case BFE_DFET_ON:
        {
            system_config_1_tmp1.value_b.LDFET_EN = 1;

            break;
        }

        case BFE_DFET_OFF:
        {
            system_config_1_tmp1.value_b.LDFET_EN = 0;

            break;
        }

        default:
        {
            return BFE_ERR_INVALID_ARGUMENT; // Error return

            break;
        }
    }

    /* Write System Config 1 Register to control FETs. */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_1, system_config_1_tmp1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Verify FETs state */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_1, &p_bfe_regs->system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    system_config_1_tmp2.value = p_bfe_regs->system_config_1.value;

    /* Compare returned FET status. */
    BFE_ERROR_RETURN(system_config_1_tmp1.value_b.LCFET_EN == system_config_1_tmp2.value_b.LCFET_STAT,
                     BFE_ERR_FET_CONTROL);
    BFE_ERROR_RETURN(system_config_1_tmp1.value_b.LDFET_EN == system_config_1_tmp2.value_b.LDFET_STAT,
                     BFE_ERR_FET_CONTROL);

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_FETsCtrl
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_HVGPIOCtrl */
/******************************************************************************************************************//**
 * This function controls the HVGPIO used for Fuse blow and battery pack disabling.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Reads System Config 1 Register.
 * - Changes HVGPIO_ASSERT bit according to input parameters.
 * - Writes back to System Config 1 Register.
 * - Reads System Config 1 Register to verify status of the HVGPIO control bit.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       ctrl_option                         Pointer to GPIO pins control structure.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_INVALID_ARGUMENT            The selected FET state was not correct.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_HVGPIOCtrl (st_bfe_ctrl_t * const p_ctrl, bfe_gpio_ctrl_t * const p_control_options)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_registers_t   * p_bfe_regs       = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;
    st_raa489220_hvgpio_ctrl_t * p_hvgpio_options = (st_raa489220_hvgpio_ctrl_t *) p_control_options;

    u_raa489220_config1_reg_t system_config_1_tmp1 = {0};
    u_raa489220_config1_reg_t system_config_1_tmp2 = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Read System Config 1 Register */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_1, &p_bfe_regs->system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Change the HVGPIO Assert bit. */
    system_config_1_tmp1.value = p_bfe_regs->system_config_1.value;

    /* Copy HVGPIO state. */
    switch(p_hvgpio_options->hvgpio_state)
    {
        case BFE_HVGPIO_LOW:
        {
            system_config_1_tmp1.value_b.HVGPIO_ASSERT = 1;

            break;
        }

        case BFE_HVGPIO_HIGH_Z:
        {
            system_config_1_tmp1.value_b.HVGPIO_ASSERT = 0;

            break;
        }

        default:
        {
            return BFE_ERR_INVALID_ARGUMENT; // Error return

            break;
        }
    }

    /* Write System Config 1 Register to control FETs. */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_1, system_config_1_tmp1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Verify FETs state */
    bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->system_config_1, &p_bfe_regs->system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    system_config_1_tmp2.value = p_bfe_regs->system_config_1.value;

    /* Compare returned HVGPIO status. */
    BFE_ERROR_RETURN(system_config_1_tmp1.value_b.HVGPIO_ASSERT != system_config_1_tmp2.value_b.HVGPIO_STAT,
                     BFE_ERR_FET_CONTROL);

    p_ctrl->is_low_power = false; // BFE has been woken up by I2C communication.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_HVGPIOCtrl
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_RegisterRead */
/******************************************************************************************************************//**
 * This function reads a register in the BFE. The obtained register value can be found in the register container.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Sends a read command.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       p_register                          Pointer to register address and data container.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_read().
 *********************************************************************************************************************/

e_bfe_err_t R_RAA489220_RegisterRead (st_bfe_ctrl_t * const p_ctrl, bfe_register_t * const p_register)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_quick_reg_t * p_target_reg = (st_raa489220_quick_reg_t *) p_register;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,     BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_register, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    bfe_err = bfe_i2c_register_read(p_ctrl, *p_target_reg->p_targer_register, &p_target_reg->data);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_RegisterRead
 *********************************************************************************************************************/

/* Function Name: R_RAA489220_RegisterWrite */
/******************************************************************************************************************//**
 * This function writes to a register in a BFE.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Sends a write command.
 *
 * @pre The BFE interface should have already been initialized!
 * @param[in]       p_ctrl                              Pointer to the control structure.
 * @param[in]       p_register                          Pointer to register address and data container.
 * @retval          BFE_SUCCESS                         No error was returned.
 * @retval          BFE_ERR_INVALID_POINTER             Input argument has invalid pointer.
 * @retval          BFE_ERR_DEVICE_NOT_INITIALIZED      The BFE interface is not initialized.
 * @retval          BFE_ERR_...                         Inherit from bfe_i2c_register_write().
 *********************************************************************************************************************/
e_bfe_err_t R_RAA489220_RegisterWrite (st_bfe_ctrl_t * const p_ctrl, bfe_register_t * const p_register)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_quick_reg_t * p_target_reg = (st_raa489220_quick_reg_t *) p_register;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl,     BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_register, BFE_ERR_INVALID_POINTER);

    /* Verify BFE initialization. */
    BFE_ERROR_RETURN(true == p_ctrl->is_initialized, BFE_ERR_DEVICE_NOT_INITIALIZED);

#endif // BFE_CFG_PARAM_CHECKING_EN

    bfe_err = bfe_i2c_register_write(p_ctrl, *p_target_reg->p_targer_register, p_target_reg->data);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function R_RAA489220_RegisterWrite
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_reset_hard
 * Description  : Pulls up THERM2 pin for more than 500ms to trigger a reset.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Changes RESET pin type from input to output.
 * - Pulls up and holds RESET pin.
 * - Reverts RESET pin type from output to input.
 * - Waits until the reset has completed.
 *
 * Arguments    : p_ctrl                    Pointer to the control structure.
 * Return Value : BFE_SUCCESS               No error was returned
 *                BFE_ERR_INVALID_POINTER   An input argument is invalid
 *                BFE_ERR_FSP               The FSP layer has returned an error
 *********************************************************************************************************************/
static e_bfe_err_t bfe_reset_hard(st_bfe_ctrl_t * const p_ctrl)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status
    fsp_err_t   fsp_err = FSP_SUCCESS; // Error status

    st_raa489220_ext_ctrl_t * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;
    st_raa489220_ext_cfg_t  * p_ext_cfg  = (st_raa489220_ext_cfg_t *) p_ctrl->p_cfg->p_extend;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointers are valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Reconfigure the reset pin from being an input to an output and set high level. */
    fsp_err = g_ioport.p_api->pinCfg(&g_ioport_ctrl,
                                     p_ext_cfg->pin_reset,
                                     IOPORT_CFG_PORT_DIRECTION_OUTPUT | IOPORT_CFG_PORT_OUTPUT_HIGH);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Wait for the BFE to detect a reset condition. */
    R_BSP_SoftwareDelay(p_ext_ctrl->timings.reset_therm2_hold_time_ms, BSP_DELAY_UNITS_MILLISECONDS);

    /* Reconfigure the reset pin from being an input to an input. */
    fsp_err = g_ioport.p_api->pinCfg(&g_ioport_ctrl, p_ext_cfg->pin_reset, IOPORT_CFG_PORT_DIRECTION_INPUT);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Wait for devices to reset. */
    R_BSP_SoftwareDelay(p_ext_ctrl->timings.reset_wait_time_ms, BSP_DELAY_UNITS_MILLISECONDS);

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function bfe_reset_hard
 *********************************************************************************************************************/


/**********************************************************************************************************************
 * Function Name: bfe_reset_soft
 * Description  : Trigger a soft reset to reset all register values to the factory settings, including data registers.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Writes System Config 1 Register to set the Soft Reset Bit.
 * - Wait for the BFE to reset and enter Idle Mode
 *
 * Arguments    : p_ctrl                    Pointer to the control structure.
 * Return Value : BFE_SUCCESS               No error was returned
 *                BFE_ERR_INVALID_POINTER   An input argument is invalid
 *                BMS_ERR_...               Inherit from bfe_i2c_register_write().
 *********************************************************************************************************************/
static e_bfe_err_t bfe_reset_soft(st_bfe_ctrl_t * const p_ctrl)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_ext_ctrl_t  * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;
    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;

    u_raa489220_config1_reg_t    system_config_1 = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointers are valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Set the Soft Reset Bit */
    system_config_1.value = 0;
    system_config_1.value_b.SOFT_RESET = 1;

    /* Write System Config 1 Register to trigger a soft reset. */
    bfe_err = bfe_i2c_register_write(p_ctrl, p_bfe_regs->system_config_1, system_config_1.value);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Wait for devices to reset. */
    R_BSP_SoftwareDelay(p_ext_ctrl->timings.reset_wait_time_ms, BSP_DELAY_UNITS_MILLISECONDS);

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function bfe_reset_soft
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_i2c_register_write
 * Description  : Writes data in a BFE register. The target registers cannot be read-only or command types.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Sets I2C slave address.
 * - Calculate CRC (optional).
 * - Copies data to output buffer and assemble data packet.
 * - Sends data via I2C interface.
 * - Validate successful data transmission.
 *
 * Arguments    : tar_reg                   The target register
 *                data                      The data to be written
 * Return Value : BFE_SUCCESS               No error was returned
 *                BFE_ERR_INVALID_ARGUMENT  An input argument is invalid
 *                BFE_ERR_FSP               The FSP layer has returned an error
 *                BFE_ERR_COMM_FAULT        The data packet was not successfully sent
 *********************************************************************************************************************/
static e_bfe_err_t bfe_i2c_register_write(st_bfe_ctrl_t * const p_ctrl,
                                          st_raa489220_register_t tar_reg,
                                          uint16_t data)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status
    fsp_err_t   fsp_err = FSP_SUCCESS; // Error status

    st_raa489220_ext_ctrl_t  * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;

    static uint32_t s_crc_res = 0;

    uint32_t bytes = 0;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify write is possible. */
    BFE_ERROR_RETURN(READ_WRITE == tar_reg.type, BFE_ERR_INVALID_ARGUMENT);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Clean buffer. */
    memset(&s_i2c_buffer[0], 0, sizeof(s_i2c_buffer));

    /* Set the I2C slave address */
    fsp_err = R_IIC_MASTER_SlaveAddressSet(&g_i2c_raa489220_ctrl,
                                           p_ext_ctrl->i2c_slave_address,
                                           I2C_MASTER_ADDR_MODE_7BIT);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Copy data to output buffer. */
    s_i2c_buffer[0] = (uint8_t) ((p_ext_ctrl->i2c_slave_address << 1) & 0xFE); // Copy slave address
    s_i2c_buffer[1] = tar_reg.address;
    s_i2c_buffer[2] = (uint8_t) (data & 0x00FF);
    s_i2c_buffer[3] = (uint8_t) ((data & 0xFF00) >> 8);

    bytes = BFE_I2C_TRANSMIT_BYTES; // Determine data packet size.

#if BFE_I2C_CRC_USE // I2C communication with CRC

    bytes += BFE_I2C_CRC_SIZE_BYTES; // Add CRC byte to the total packet size

    /* Calculate the CRC*/
    bfe_err = bfe_crc8_calculate(&s_i2c_buffer[0],
                                 bytes - BFE_I2C_CRC_SIZE_BYTES + BFE_I2C_ADDR_SIZE_BYTES,
                                 &s_crc_res);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    s_i2c_buffer[4] = (uint8_t) s_crc_res;   // Add checksum to the output buffer

#else

    FSP_PARAMETER_NOT_USED(s_crc_res); // Mute unused parameter

#endif // BFE_I2C_CRC_USE

    s_i2c_callback_event = (i2c_master_event_t) RESET_VALUE; // Clear I2C event.

    /* Send data through I2C interface. */
    fsp_err = g_i2c_raa489220.p_api->write(&g_i2c_raa489220_ctrl, &s_i2c_buffer[1], bytes, false);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Wait until data is sent. */
    fsp_err = bfe_i2c_event_validate();
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_COMM_FAULT); // Check for errors.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function bfe_i2c_register_write
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_i2c_register_read
 * Description  : Read data from a BFE register. The target registers cannot be command type.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Sets I2C slave address.
 * - Sends target address via I2C interface.
 * - Validate successful data transmission.
 * - Receives requested data via I2C interface.
 * - Validate successful data reception.
 * - Calculate and compare CRC (optional).
 * - Copy received data
 *
 * Arguments    : tar_reg                   The target register
 *                p_data                    Pointer to the read data
 * Return Value : BFE_SUCCESS               No error was returned
 *                BFE_ERR_INVALID_POINTER   An input pointer is invalid
 *                BFE_ERR_INVALID_ARGUMENT  An input argument is invalid
 *                BFE_ERR_FSP               The FSP layer has returned an error
 *                BFE_ERR_COMM_FAULT        The data packet was not successfully sent
 *                BFE_ERR_CRC_INCORRECT     The CRC of the received packet does not match
 *********************************************************************************************************************/
static e_bfe_err_t bfe_i2c_register_read(st_bfe_ctrl_t * const p_ctrl,
                                         st_raa489220_register_t tar_reg,
                                         uint16_t * const p_data)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status
    fsp_err_t   fsp_err = FSP_SUCCESS; // Error status

    st_raa489220_ext_ctrl_t  * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;

    static uint32_t s_crc_res = 0;

    uint32_t bytes = 0;
    uint32_t tmp1  = 0;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the input arguments are valid. */
    BFE_ERROR_RETURN(NULL != p_data,  BFE_ERR_INVALID_POINTER);

    /* Verify not command. */
    BFE_ERROR_RETURN(COMMAND != tar_reg.type, BFE_ERR_INVALID_ARGUMENT);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Clean buffers. */
    memset(&s_i2c_buffer[0], 0, sizeof(s_i2c_buffer));

    /* Set the I2C slave address */
    fsp_err = R_IIC_MASTER_SlaveAddressSet(&g_i2c_raa489220_ctrl,
                                           p_ext_ctrl->i2c_slave_address,
                                           I2C_MASTER_ADDR_MODE_7BIT);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Assemble transfer data packet */
    s_i2c_buffer[0] = (uint8_t) ((p_ext_ctrl->i2c_slave_address << 1) & 0xFE); // Copy slave address
    s_i2c_buffer[1] = tar_reg.address; // Copy target register address
    s_i2c_buffer[2] = (uint8_t) ((p_ext_ctrl->i2c_slave_address << 1) | 0x01); // Copy slave address with read bit set

    s_i2c_callback_event = (i2c_master_event_t) RESET_VALUE; // Clear i2c event.

    /* Send data through I2C interface. */
    fsp_err = g_i2c_raa489220.p_api->write(&g_i2c_raa489220_ctrl, &s_i2c_buffer[1], BFE_I2C_ADDR_SIZE_BYTES, true);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Wait until data is sent. */
    fsp_err = bfe_i2c_event_validate();
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_COMM_FAULT); // Check for errors.

    bytes = BFE_I2C_RECEIVE_BYTES;  // Determine data packet size.

#if BFE_I2C_CRC_USE // I2C communication with CRC

    bytes += BFE_I2C_CRC_SIZE_BYTES; // Add CRC byte to the total packet size

#endif // BFE_I2C_CRC_USE

    /* Read data through I2C interface. */
    fsp_err = g_i2c_raa489220.p_api->read(&g_i2c_raa489220_ctrl, &s_i2c_buffer[3], bytes, false);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Wait until data is received. */
    fsp_err = bfe_i2c_event_validate();
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_COMM_FAULT); // Check for errors.

#if BFE_I2C_CRC_USE // I2C communication with CRC

    /* Calculate the CRC*/
    tmp1 = bytes - BFE_I2C_CRC_SIZE_BYTES + BFE_I2C_ADDR_HEADER_SIZE_BYTES;
    bfe_err = bfe_crc8_calculate(&s_i2c_buffer[0], tmp1, &s_crc_res);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Compare the received and calculated checksums. */
    BFE_ERROR_RETURN(s_i2c_buffer[5] == s_crc_res, BFE_ERR_CRC_INCORRECT);

#else

    FSP_PARAMETER_NOT_USED(s_crc_res); // Mute unused parameter
    FSP_PARAMETER_NOT_USED(tmp1); // Mute unused parameter

#endif // BFE_I2C_CRC_USE

    /* Copy the received data. */

    *p_data = (uint16_t) (s_i2c_buffer[3] | (0xFF00 & (s_i2c_buffer[4] << 8)));

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function bfe_i2c_register_read
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_i2c_block_read
 * Description  : Bulk read data from a BFE register. The target registers must be a command type.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Sets I2C slave address.
 * - Sends target command via I2C interface.
 * - Validate successful data transmission.
 * - Receives requested data via I2C interface.
 * - Validate successful data reception.
 * - Calculate and compare CRC (optional).
 * - Copy received data to output data buffer
 *
 * Warning      : Make sure that the data buffer is big enough!
 * Arguments    : tar_reg                   The target register
 *                p_data                    Pointer to the read data
 * Return Value : BFE_SUCCESS               No error was returned
 *                BFE_ERR_INVALID_POINTER   An input pointer is invalid
 *                BFE_ERR_INVALID_ARGUMENT  An input argument is invalid
 *                BFE_ERR_FSP               The FSP layer has returned an error
 *                BFE_ERR_COMM_FAULT        The data packet was not successfully sent
 *                BFE_ERR_CRC_INCORRECT     The CRC of the received packet does not match
 *********************************************************************************************************************/
static e_bfe_err_t bfe_i2c_block_read(st_bfe_ctrl_t * const p_ctrl,
                                      st_raa489220_register_t tar_reg,
                                      uint16_t * const p_data)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status
    fsp_err_t   fsp_err = FSP_SUCCESS; // Error status

    st_raa489220_ext_ctrl_t  * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;

    static uint32_t s_crc_res = 0;

    uint32_t bytes = 0;
    uint32_t tmp1  = 0;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the input arguments are valid. */
    BFE_ERROR_RETURN(NULL != p_data,  BFE_ERR_INVALID_POINTER);

    /* Verify command. */
    BFE_ERROR_RETURN(COMMAND == tar_reg.type, BFE_ERR_INVALID_ARGUMENT);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Clean buffers. */
    memset(&s_i2c_buffer[0], 0, sizeof(s_i2c_buffer));

    /* Set the I2C slave address */
    fsp_err = R_IIC_MASTER_SlaveAddressSet(&g_i2c_raa489220_ctrl,
                                           p_ext_ctrl->i2c_slave_address,
                                           I2C_MASTER_ADDR_MODE_7BIT);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Determine bulk read data length. */
    switch(tar_reg.address)
    {
        case BFE_FAULT_AND_STATUS:
        {
            bytes = BFE_I2C_RECEIVE_CMND_FS_BYTES;

            break;
        }

        case BFE_READ_MEASUREMENTS:
        {
            bytes = BFE_I2C_RECEIVE_CMND_RM_BYTES;

            break;
        }

        case BFE_I_PACK:
        {
            bytes = BFE_I2C_RECEIVE_CMND_IP_BYTES;

            break;
        }

        default:
        {
            bfe_err = BFE_ERR_INVALID_ARGUMENT; // Error return

            break;
        }
    }

#if BFE_I2C_CRC_USE // I2C communication with CRC

    bytes += BFE_I2C_CRC_SIZE_BYTES;  // Determine data packet size.

#endif // BFE_I2C_CRC_USE

    /* Assemble transfer data packet */
    s_i2c_buffer[0] = (uint8_t) ((p_ext_ctrl->i2c_slave_address << 1) & 0xFE); // Copy slave address
    s_i2c_buffer[1] = tar_reg.address; // Copy target register address
    s_i2c_buffer[2] = (uint8_t) ((p_ext_ctrl->i2c_slave_address << 1) | 0x01); // Copy slave address with read bit set

    s_i2c_callback_event = (i2c_master_event_t) RESET_VALUE; // Clear I2C event.

    /* Send data through I2C interface. */
    fsp_err = g_i2c_raa489220.p_api->write(&g_i2c_raa489220_ctrl, &s_i2c_buffer[1], BFE_I2C_ADDR_SIZE_BYTES, true);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Wait until data is sent. */
    fsp_err = bfe_i2c_event_validate();
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_COMM_FAULT); // Check for errors.

    /* Read data through I2C interface. */
    fsp_err = g_i2c_raa489220.p_api->read(&g_i2c_raa489220_ctrl, &s_i2c_buffer[3], bytes, false);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    /* Wait until data is sent. */
    fsp_err = bfe_i2c_event_validate();
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_COMM_FAULT); // Check for errors.

#if BFE_I2C_CRC_USE // I2C communication with CRC

    /* Calculate the CRC*/
    tmp1 = bytes - BFE_I2C_CRC_SIZE_BYTES + BFE_I2C_ADDR_HEADER_SIZE_BYTES;
    bfe_err = bfe_crc8_calculate(&s_i2c_buffer[0], tmp1, &s_crc_res);
    BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

    /* Compare the received and calculated checksums. */
    BFE_ERROR_RETURN(s_i2c_buffer[tmp1] == s_crc_res, BFE_ERR_CRC_INCORRECT);

#else

        FSP_PARAMETER_NOT_USED(s_crc_res); // Mute unused parameter
        FSP_PARAMETER_NOT_USED(tmp1); // Mute unused parameter

#endif // BFE_I2C_CRC_USE

    /* Copy received data. */
    for(uint16_t i = 0; i < (bytes / 2); i++ )
    {
        *(p_data + i) = (uint16_t) (s_i2c_buffer[(i * 2) + 3U] | (0xFF00 & (s_i2c_buffer[(i * 2) + 4U] << 8)));
    }

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function bfe_i2c_block_read
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_i2c_event_validate
 * Description  : Validate that the I2C transfer has completed or was aborted.
 *
 * This function performs the following tasks:
 * - Calculate a timeout to prevent sticking into the loop.
 * - Check the I2C callback event status.
 *
 * Return Value : FSP_SUCCESS               No error was returned
 *                FSP_ERR_TIMEOUT           An input pointer is invalid
 *                FSP_ERR_TRANSFER_ABORTED  An input argument is invalid
 *********************************************************************************************************************/
static fsp_err_t bfe_i2c_event_validate(void)
{
    fsp_err_t fsp_err = FSP_SUCCESS;

    uint16_t timeout_ms = UINT16_MAX;

    /* Protect from infinite looping. */
    do
    {
        --timeout_ms;

        if(0 == timeout_ms)
        {
            return FSP_ERR_TIMEOUT;
        }
    }
    while(RESET_VALUE == s_i2c_callback_event);

    /* Check if communication was aborted. */
    if(s_i2c_callback_event == I2C_MASTER_EVENT_ABORTED)
    {
        s_i2c_callback_event = (i2c_master_event_t) RESET_VALUE;  // Make sure this is always Reset before return

        return FSP_ERR_TRANSFER_ABORTED;
    }

    s_i2c_callback_event = (i2c_master_event_t) RESET_VALUE;  // Make sure this is always Reset before return

    return fsp_err;
}
/**********************************************************************************************************************
 * End of function bfe_i2c_event_validate
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_crc8_calculate
 * Description  : Calculate CRC8 checksum.
 *
 * This function performs the following tasks:
 * - Checks parameters and processes errors (optional).
 * - Calculate CRC8 checksum.
 *
 * Arguments    : p_data_buffer             Pointer to input data buffer
 *                length                    Number of Bytes
 *                p_crc_result              Pointer to result
 * Return Value : BFE_SUCCESS               No error was returned
 *                BFE_ERR_INVALID_POINTER   An input pointer is invalid
 *                BFE_ERR_INVALID_ARGUMENT  An input argument is invalid
 *                BFE_ERR_FSP               The FSP layer has returned an error
 *********************************************************************************************************************/
static e_bfe_err_t bfe_crc8_calculate(uint8_t * const p_data_buffer, uint32_t length, uint32_t * const p_crc_result)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status
    fsp_err_t   fsp_err = FSP_SUCCESS; // Error status

    static crc_input_t s_crc_input = {0};

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the input arguments are valid. */
    BFE_ERROR_RETURN(NULL != p_data_buffer,  BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(NULL != p_crc_result,  BFE_ERR_INVALID_POINTER);
    BFE_ERROR_RETURN(0 != length, BFE_ERR_INVALID_ARGUMENT);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Set input parameters of calculate CRC function */
    s_crc_input.p_input_buffer = p_data_buffer;
    s_crc_input.crc_seed = 0;
    s_crc_input.num_bytes = length;

    /* Calculate the CRC*/
    fsp_err = g_crc8.p_api->calculate(&g_crc8_ctrl, &s_crc_input, p_crc_result);
    BFE_ERROR_RETURN(FSP_SUCCESS == fsp_err, BFE_ERR_FSP); // Check for errors.

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function bfe_crc8_calculate
 *********************************************************************************************************************/


/**********************************************************************************************************************
 * Function Name: bfe_meas_complete_wait
 * Description  : Wait until measurement has completed.
 *
 * Arguments    : p_ctrl                    Pointer to the control structure.
 *                selection                 Measurement selection
 * Return Value : BFE_SUCCESS               No error was returned.
 *                BFE_ERR_MEAS_TIMEOUT      The Busy bit was not cleared in time.
 *                BFE_ERR_INVALID_ARGUMENT  An input argument is invalid.
 *                BFE_ERR_...               Inherit from bfe_i2c_register_read().
 *********************************************************************************************************************/
static e_bfe_err_t bfe_meas_complete_wait(st_bfe_ctrl_t * const p_ctrl, const e_raa489220_meas_select_t selection)
{
    e_bfe_err_t bfe_err = BFE_SUCCESS; // Error status

    st_raa489220_ext_ctrl_t  * p_ext_ctrl = (st_raa489220_ext_ctrl_t *) p_ctrl->p_extend;
    st_raa489220_registers_t * p_bfe_regs = (st_raa489220_registers_t *) p_ctrl->p_bfe_regs;

    u_raa489220_meas_sel_reg_t measure_select = {0};

    uint16_t busy_wait_count = BFE_BUSY_MAX_CNT;

#if BFE_CFG_PARAM_CHECKING_EN // Perform parameter checking.

    /* Verify the pointer is valid. */
    BFE_ERROR_RETURN(NULL != p_ctrl, BFE_ERR_INVALID_POINTER);

#endif // BFE_CFG_PARAM_CHECKING_EN

    /* Wait until Busy bit is cleared */
    do
    {
        /* Communication timeout */
        R_BSP_SoftwareDelay(p_ext_ctrl->timings.busy_bit_timeout_us, BSP_DELAY_UNITS_MICROSECONDS);

        /* Safety timeout operation, if busy bit is not cleared. */
        busy_wait_count--;

        /* Check for error conditions. */
        BFE_ERROR_RETURN(BFE_BUSY_MIN_CNT < busy_wait_count, BFE_ERR_MEAS_TIMEOUT);

        /* Read Measure Select Register */
        bfe_err = bfe_i2c_register_read(p_ctrl, p_bfe_regs->measure_select, &s_data_buffer[0]);
        BFE_ERROR_RETURN(BFE_SUCCESS == bfe_err, bfe_err); // Check for errors.

        measure_select.value = s_data_buffer[0];

#if BFE_CFG_REG_WRITE_VERIFY_EN // Verify register write

        /* Check measurement selection */
        BFE_ERROR_RETURN(measure_select.value_b.MEAS_SELECT == selection, BFE_ERR_WRITE_VERIFY);

#else

        FSP_PARAMETER_NOT_USED(selection); // Mute unused parameter

#endif // BFE_CFG_REG_WRITE_VERIFY_EN

    } while(measure_select.value_b.BUSY == 1);  // Test the Busy bit

    return bfe_err;
}
/**********************************************************************************************************************
 * End of function bfe_meas_complete_wait
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_adc_to_vpack
 * Description  : Convert ADC value to pack voltage.
 * Arguments    : value     ADC value [16bit raw]
 * Return Value : volt      Pack voltage [V]
 *********************************************************************************************************************/
static float bfe_adc_to_vpack(uint16_t value)
{
    float volt = 0;
    float meas_step = 0.02f;
    float meas_min = -5.12f;

    volt = (((float) value) * meas_step) + (0.5f * meas_step) + meas_min;

    return volt;
}
/**********************************************************************************************************************
 * End of function bfe_adc_to_vpack
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_adc_to_ipack
 * Description  : Convert ADC value to pack current.
 * Arguments    : value         ADC value [16bit raw]
 *                r_shunt_mohm  Shunt resistance [mohms]
 * Return Value : volt          Pack current [A]
 *********************************************************************************************************************/
static float bfe_adc_to_ipack(uint16_t value, float r_shunt_mohm)
{
    float current = 0;
    float meas_step = 0.0132f;
    float meas_min = -3.368f;
    float voltage = 0;

    //current = ((((float) value) * meas_step) + (0.5f * meas_step) + meas_min) / r_shunt_mohm;

    voltage = ((((float) value) * meas_step) + (0.5f * meas_step) + meas_min);
    current = voltage / r_shunt_mohm;

    return current;
}
/**********************************************************************************************************************
 * End of function bfe_adc_to_ipack
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_adc_to_vcell
 * Description  : Convert ADC value to cell voltage.
 * Arguments    : value         ADC value [16bit raw]
 * Return Value : volt          Cell voltage [V]
 *********************************************************************************************************************/
static float bfe_adc_to_vcell(uint16_t value)
{
    float volt = 0;
    float meas_step = 0.001f;
    float meas_min = 0.512f;

    volt = (((float) value) * meas_step) + (0.5f * meas_step) + meas_min;

    return volt;
}
/**********************************************************************************************************************
 * End of function bfe_adc_to_vcell
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_adc_to_vtherm
 * Description  : Convert ADC value to thermistor voltage.
 * Arguments    : value         ADC value [16bit raw]
 * Return Value : volt          Thremistor voltage [V]
 *********************************************************************************************************************/
static float bfe_adc_to_vtherm(uint16_t value)
{
    float volt = 0;
    float meas_step = 0.0005f;
    float meas_min = -0.128f;

    volt = (((float) value) * meas_step) + (0.5f * meas_step) + meas_min;

    return volt;
}
/**********************************************************************************************************************
 * End of function bfe_adc_to_vtherm
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_adc_to_vregpin
 * Description  : Convert ADC value to 3.3V and 2.5V regulator voltage.
 * Arguments    : value         ADC value [16bit raw]
 * Return Value : volt          Regulator pin voltage [V]
 *********************************************************************************************************************/
static float bfe_adc_to_vregpin(uint16_t value)
{
    float volt = 0;
    float meas_step = 0.001f;
    float meas_min = 0.512f;

    volt = (((float) value) * meas_step) + (0.5f * meas_step) + meas_min;

    return volt;
}
/**********************************************************************************************************************
 * End of function bfe_adc_to_vregpin
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: bfe_adc_to_vbg2ref
 * Description  : Convert ADC value to secondary Band Gap reference voltage.
 * Arguments    : value         ADC value [16bit raw]
 * Return Value : volt          Secondary Band Gap reference voltage [V]
 *********************************************************************************************************************/
static float bfe_adc_to_vbg2ref(uint16_t value)
{
    float volt = 0;
    float meas_step = 0.0005f;
    float meas_min = -0.128f;

    volt = (((float) value) * meas_step) + (0.5f * meas_step) + meas_min;

    return volt;
}
/**********************************************************************************************************************
 * End of function bfe_adc_to_vbg2ref
 *********************************************************************************************************************/

/******************************************************************************************************************//**
 * I2C callback function.
 * @param[in] p_args
 * @retval None
 *********************************************************************************************************************/
void i2c_callback (i2c_master_callback_args_t * p_args)
{
    if (NULL != p_args)
    {
        s_i2c_callback_event = p_args->event;
    }
    else
    {
        ; // Do nothing.
    }
}
/**********************************************************************************************************************
 * End of function spi_isr_callback
 *********************************************************************************************************************/

/******************************************************************************************************************//**
 * Alert callback function.
 * @param[in] p_args
 * @retval None
 *********************************************************************************************************************/
void alert_irq_callback(external_irq_callback_args_t * p_args)
{
    /* Check the external interrupt channel. */
    if (g_alert_irq_cfg.channel == p_args->channel)
    {
        s_alert_event_flag = true;
    }
    else
    {
        ; // Do nothing.
    }
}
/**********************************************************************************************************************
 * End of function spi_callback
 *********************************************************************************************************************/
