/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2021 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/******************************************************************************************************************//**
 * @file          r_bms.h
 * @Version       1.0
 * @brief         Contains macros and declarations of functions used in r_bms.c
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 11.07.2022 1.00     First Release.
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/

/* Generic headers */
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* BFE related headers */
#include "bal_data.h"
#include "board_cfg.h"

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/
#ifndef BMS_H_
#define BMS_H_

typedef enum data_bits
{
    DATA_BITS_5 = 5U,
    DATA_BITS_6,
    DATA_BITS_7,
    DATA_BITS_8,
    DATA_BITS_16 = 16U,
}data_bits_t;

typedef enum parity_type
{
    PARITY_NONE = 0,
    PARITY_ODD,
    PARITY_EVEN,
    PARITY_MARK,
    PARITY_SPACE,
}parity_type_t;

typedef enum stop_bits
{
    STOP_BITS_1 = 0,
    STOP_BITS_1_5,
    STOP_BITS_2,
}stop_bits_t;

#define NUM_STRING_DESCRIPTOR               (7U)

/* Convert float numbers macros */
#define FL_INT(a)                   (((int) (a)))
#define FL_FRAC_V(a)                (((int) ((a - (float) ((int) (a))) * 100.0f)))
#define FL_FRAC_I(a)                (((int) ((a - (float) ((int) (a))) * 1000.0f)))


/* USB communication buffers */
#define USB_READ_BUF_SIZE           (1U)
#define USB_SHORT_PACKET_SIZE       (256U)
#define USB_LONG_PACKET_SIZE        (512U)

#define AUTO_SCAN_RESET             (BMS_AUTO_SCAN - 10U)

/* User interface menu positions */
#define GET_VOLT_TEMP               ('1')
#define FETS_CTRL                   ('2')
#define AUTOSCAN                    ('3')
#define SLEEP                       ('4')
#define MORE_INFO                   ('5')
#define CARRIAGE_RETURN             ('\r')

/* Banners */
#define INTRO               "\r\n\r\n\r\n Welcome to RAA489220 sample project for BFE library v.1.0!"

#define MAIN_MENU           "\r\n\r\n Press 1 for voltage and temperature reading.\r\n" \
                            "\r\n Press 2 for turning ON the Charge and Discharge FETs.\r\n" \
                            "\r\n Press 3 for entering Auto Scan Mode.\r\n" \
                            "\r\n Press 4 for entering Low Power Mode.\r\n" \
                            "\r\n Press 5 for more info.\r\n"

#define MAIN_MENU_2         "\r\n\r\n Press 1 for voltage and temperature reading.\r\n" \
                            "\r\n Press 2 for turning OFF the Charge and Discharge FETs.\r\n" \
                            "\r\n Press 3 for entering Auto Scan Mode.\r\n" \
                            "\r\n Press 4 for entering Low Power Mode.\r\n" \
                            "\r\n Press 5 for more info.\r\n"

#define FETS_CTRL_1         "\r\n\r\n The Charge and Discharge FETs are OFF!\r\n"

#define FETS_CTRL_2         "\r\n\r\n The Charge and Discharge FETs are ON!\r\n"

#define AUTOSCAN_MODE_1     "\r\n\r\n 2. AUTO SCAN MODE " \

#define AUTOSCAN_MODE_2     "\r\n\r\n The Battery Front End is in continuously scanning!" \
                            "\r\n\r\n Press 'Enter' to exit!\r\n"

#define AUTOSCAN_MODE_3     "\r\n\r\n Auto Scan Mode is Inhibited!"

#define SLEEP_MODE_1        "\r\n\r\n 3. LOW POWER MODE \r\n "

#define SLEEP_MODE_2        "\r\n\r\n The Battery Front End is in Low Power Mode!" \
                            "\r\n\r\n Press 'Enter' to exit!\r\n"

#define SLEEP_MODE_3        "\r\n\r\n The Battery Front End is in Idle Mode!"

#define INFO_BANNER_1       "\r\n\r\n 4. MORE INFO \r\n" \
                            "\r\nVisit the following URLs to learn about the setup, " \
                            "the Battery Front End ICs and the RA family of MCUs, download tools " \
                            "and documentation, and get support:\r\n" \
                            "\r\n a) RTKA489220DE0000BU resources: \t renesas.com/RTKA489220DE0000BU" \
                            "\r\n b) EK-RA2A1 resources: \t\t renesas.com/ek-ra2a1" \
                            "\r\n c) BFE product information:\t\t renesas.com/raa489220" \
                            "\r\n d) Renesas support: \t\t\t renesas.com/support" \
                            "\r\n\r\n Press 'Enter' to return to menu...\r\n"

#define FAULTS_MODE         "\r\n\r\n FAULT DETECTED! \r\n "

#define FAULTS_MSG_1        "\r\n\r\n Critical fault! Please, restart! "

#define FAULTS_MSG_2        "\r\n\r\n Communication error! " \
                            "\r\n\r\n Press 'Enter' to restart the BFE.."

#define FAULTS_MSG_3        "\r\n\r\n Fault clear success!!!\r\n\r\n"

#define MEAS_MSG_1          "\r\n\r\n 1. VOLTAGE AND TEMPERATURE READINGS \r\n "
#define MEAS_MSG_2          "\r\n\r\n Battery pack voltage [V]: "
#define MEAS_MSG_3          "\r\n Battery pack discharge current [A]: "
#define MEAS_MSG_4          "\r\n Battery pack charge current [A]: "
#define MEAS_MSG_5          "\r\n Maximum cell voltage [V]: "
#define MEAS_MSG_6          "\r\n Minimum cell voltage [V]: "

#define TABLE_HEADER_TEMPS  "\r\n|  Thermistor input [V]   |" \
                            "\r\n|     #1     |     #2     |\r\n|"

#define TABLE_HEADER_CVOLTS "\r\n|                          Cell voltage [V]                           |" \
                            "\r\n|  #1  |  #2  |  #3  |  #4  |  #5  |  #6  |  #7  |  #8  |  #9  |  #10 |\r\n|"

#define LINE_CODING_LENGTH (0x07U)

/* Trap the error location */
#define APP_ERR_TRAP(a)             if(a) {__BKPT(0);}

/**********************************************************************************************************************
 Global Typedef definitions
 *********************************************************************************************************************/

/** Common app error codes */
typedef enum
{
    BMS_SUCCESS                         = 0,    ///< BMS_SUCCESS

    /* Start of BFE specific. */
    BMS_ERR_BFE                         = 10,   ///< Error in the BFE library

    /* Start of FSP specific. */
    BMS_ERR_FSP                         = 61,   ///< Error in FSP layer

} e_bms_err_t;

/** Battery management system states */
typedef enum
{
    BMS_INIT_STATE = 0,
    BMS_IDLE_MODE,
    BMS_AUTOSCAN_MODE,
    BMS_SCAN_STATE,
    BMS_SLEEP_MODE,
    BMS_FAULT_MODE
} e_bms_machine_states_t;

/** Battery management system state transition flags. */
typedef struct
{
    bool goto_idle;
    bool goto_measure;
    bool goto_autoscan;
    bool goto_sleep;
    bool goto_fault;
} st_bms_trans_flags_t;

/**********************************************************************************************************************
 External global variables
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global functions
 *********************************************************************************************************************/

/* Function Name: r_bms_main */
/******************************************************************************************************************//**
 * @brief Main function for the battery management demo.
 *********************************************************************************************************************/
void r_bms_main(void);

#endif /* BMS_H_ */
